CREATE TEMP FUNCTION udf_main_summary_scalars(processes ANY TYPE) AS (
  STRUCT(
    processes.parent.scalars.a11y_backplate AS scalar_parent_a11y_backplate,
    processes.parent.scalars.a11y_indicator_acted_on AS scalar_parent_a11y_indicator_acted_on,
    processes.parent.scalars.a11y_instantiators AS scalar_parent_a11y_instantiators,
    processes.parent.scalars.aushelper_websense_reg_version AS scalar_parent_aushelper_websense_reg_version,
    processes.parent.scalars.blocklist_last_modified_rs_addons AS scalar_parent_blocklist_lastModified_rs_addons,
    processes.parent.scalars.blocklist_last_modified_rs_plugins AS scalar_parent_blocklist_lastModified_rs_plugins,
    processes.parent.scalars.blocklist_last_modified_xml AS scalar_parent_blocklist_lastModified_xml,
    processes.parent.scalars.blocklist_use_xml AS scalar_parent_blocklist_useXML,
    processes.parent.scalars.browser_engagement_active_ticks AS scalar_parent_browser_engagement_active_ticks,
    processes.parent.scalars.browser_engagement_max_concurrent_tab_count AS scalar_parent_browser_engagement_max_concurrent_tab_count,
    processes.parent.scalars.browser_engagement_max_concurrent_tab_pinned_count AS scalar_parent_browser_engagement_max_concurrent_tab_pinned_count,
    processes.parent.scalars.browser_engagement_max_concurrent_window_count AS scalar_parent_browser_engagement_max_concurrent_window_count,
    processes.parent.scalars.browser_engagement_restored_pinned_tabs_count AS scalar_parent_browser_engagement_restored_pinned_tabs_count,
    processes.parent.scalars.browser_engagement_tab_open_event_count AS scalar_parent_browser_engagement_tab_open_event_count,
    processes.parent.scalars.browser_engagement_tab_pinned_event_count AS scalar_parent_browser_engagement_tab_pinned_event_count,
    processes.parent.scalars.browser_engagement_total_uri_count AS scalar_parent_browser_engagement_total_uri_count,
    processes.parent.scalars.browser_engagement_unfiltered_uri_count AS scalar_parent_browser_engagement_unfiltered_uri_count,
    processes.parent.scalars.browser_engagement_unique_domains_count AS scalar_parent_browser_engagement_unique_domains_count,
    processes.parent.scalars.browser_engagement_window_open_event_count AS scalar_parent_browser_engagement_window_open_event_count,
    processes.parent.scalars.browser_errors_collected_count AS scalar_parent_browser_errors_collected_count,
    processes.parent.scalars.browser_errors_collected_with_stack_count AS scalar_parent_browser_errors_collected_with_stack_count,
    processes.parent.scalars.browser_errors_reported_failure_count AS scalar_parent_browser_errors_reported_failure_count,
    processes.parent.scalars.browser_errors_reported_success_count AS scalar_parent_browser_errors_reported_success_count,
    processes.parent.scalars.browser_errors_sample_rate AS scalar_parent_browser_errors_sample_rate,
    processes.parent.scalars.browser_feeds_feed_subscribed AS scalar_parent_browser_feeds_feed_subscribed,
    processes.parent.scalars.browser_feeds_livebookmark_count AS scalar_parent_browser_feeds_livebookmark_count,
    processes.parent.scalars.browser_feeds_livebookmark_item_opened AS scalar_parent_browser_feeds_livebookmark_item_opened,
    processes.parent.scalars.browser_feeds_livebookmark_opened AS scalar_parent_browser_feeds_livebookmark_opened,
    processes.parent.scalars.browser_feeds_preview_loaded AS scalar_parent_browser_feeds_preview_loaded,
    processes.parent.scalars.browser_searchinit_engines_cache_corrupted AS scalar_parent_browser_searchinit_engines_cache_corrupted,
    processes.parent.scalars.browser_session_restore_browser_startup_page AS scalar_parent_browser_session_restore_browser_startup_page,
    processes.parent.scalars.browser_session_restore_browser_tabs_restorebutton AS scalar_parent_browser_session_restore_browser_tabs_restorebutton,
    processes.parent.scalars.browser_session_restore_number_of_tabs AS scalar_parent_browser_session_restore_number_of_tabs,
    processes.parent.scalars.browser_session_restore_number_of_win AS scalar_parent_browser_session_restore_number_of_win,
    processes.parent.scalars.browser_session_restore_tabbar_restore_available AS scalar_parent_browser_session_restore_tabbar_restore_available,
    processes.parent.scalars.browser_session_restore_tabbar_restore_clicked AS scalar_parent_browser_session_restore_tabbar_restore_clicked,
    processes.parent.scalars.browser_session_restore_worker_restart_count AS scalar_parent_browser_session_restore_worker_restart_count,
    processes.parent.scalars.browser_timings_last_shutdown AS scalar_parent_browser_timings_last_shutdown,
    processes.parent.scalars.browser_usage_graphite AS scalar_parent_browser_usage_graphite,
    processes.parent.scalars.browser_usage_plugin_instantiated AS scalar_parent_browser_usage_plugin_instantiated,
    processes.parent.scalars.contentblocking_category AS scalar_parent_contentblocking_category,
    processes.parent.scalars.contentblocking_cryptomining_blocking_enabled AS scalar_parent_contentblocking_cryptomining_blocking_enabled,
    processes.parent.scalars.contentblocking_enabled AS scalar_parent_contentblocking_enabled,
    processes.parent.scalars.contentblocking_exceptions AS scalar_parent_contentblocking_exceptions,
    processes.parent.scalars.contentblocking_fastblock_enabled AS scalar_parent_contentblocking_fastblock_enabled,
    processes.parent.scalars.contentblocking_fingerprinting_blocking_enabled AS scalar_parent_contentblocking_fingerprinting_blocking_enabled,
    processes.parent.scalars.contentblocking_trackers_blocked_count AS scalar_parent_contentblocking_trackers_blocked_count,
    processes.parent.scalars.corroborate_omnijar_corrupted AS scalar_parent_corroborate_omnijar_corrupted,
    processes.parent.scalars.corroborate_system_addons_corrupted AS scalar_parent_corroborate_system_addons_corrupted,
    processes.parent.scalars.devtools_aboutdevtools_installed AS scalar_parent_devtools_aboutdevtools_installed,
    processes.parent.scalars.devtools_aboutdevtools_noinstall_exits AS scalar_parent_devtools_aboutdevtools_noinstall_exits,
    processes.parent.scalars.devtools_aboutdevtools_opened AS scalar_parent_devtools_aboutdevtools_opened,
    processes.parent.scalars.devtools_accessibility_accessible_context_menu_opened AS scalar_parent_devtools_accessibility_accessible_context_menu_opened,
    processes.parent.scalars.devtools_accessibility_node_inspected_count AS scalar_parent_devtools_accessibility_node_inspected_count,
    processes.parent.scalars.devtools_accessibility_opened_count AS scalar_parent_devtools_accessibility_opened_count,
    processes.parent.scalars.devtools_accessibility_picker_used_count AS scalar_parent_devtools_accessibility_picker_used_count,
    processes.parent.scalars.devtools_accessibility_service_enabled_count AS scalar_parent_devtools_accessibility_service_enabled_count,
    processes.parent.scalars.devtools_application_opened_count AS scalar_parent_devtools_application_opened_count,
    processes.parent.scalars.devtools_changesview_contextmenu AS scalar_parent_devtools_changesview_contextmenu,
    processes.parent.scalars.devtools_changesview_contextmenu_copy AS scalar_parent_devtools_changesview_contextmenu_copy,
    processes.parent.scalars.devtools_changesview_contextmenu_copy_declaration AS scalar_parent_devtools_changesview_contextmenu_copy_declaration,
    processes.parent.scalars.devtools_changesview_contextmenu_copy_rule AS scalar_parent_devtools_changesview_contextmenu_copy_rule,
    processes.parent.scalars.devtools_changesview_copy AS scalar_parent_devtools_changesview_copy,
    processes.parent.scalars.devtools_changesview_copy_all_changes AS scalar_parent_devtools_changesview_copy_all_changes,
    processes.parent.scalars.devtools_changesview_copy_rule AS scalar_parent_devtools_changesview_copy_rule,
    processes.parent.scalars.devtools_changesview_opened_count AS scalar_parent_devtools_changesview_opened_count,
    processes.parent.scalars.devtools_copy_full_css_selector_opened AS scalar_parent_devtools_copy_full_css_selector_opened,
    processes.parent.scalars.devtools_copy_unique_css_selector_opened AS scalar_parent_devtools_copy_unique_css_selector_opened,
    processes.parent.scalars.devtools_copy_xpath_opened AS scalar_parent_devtools_copy_xpath_opened,
    processes.parent.scalars.devtools_grid_gridinspector_opened AS scalar_parent_devtools_grid_gridinspector_opened,
    processes.parent.scalars.devtools_grid_show_grid_areas_overlay_checked AS scalar_parent_devtools_grid_showGridAreasOverlay_checked,
    processes.parent.scalars.devtools_grid_show_grid_line_numbers_checked AS scalar_parent_devtools_grid_showGridLineNumbers_checked,
    processes.parent.scalars.devtools_grid_show_infinite_lines_checked AS scalar_parent_devtools_grid_showInfiniteLines_checked,
    processes.parent.scalars.devtools_inspector_element_picker_used AS scalar_parent_devtools_inspector_element_picker_used,
    processes.parent.scalars.devtools_inspector_node_selection_count AS scalar_parent_devtools_inspector_node_selection_count,
    processes.parent.scalars.devtools_layout_flexboxhighlighter_opened AS scalar_parent_devtools_layout_flexboxhighlighter_opened,
    processes.parent.scalars.devtools_markup_flexboxhighlighter_opened AS scalar_parent_devtools_markup_flexboxhighlighter_opened,
    processes.parent.scalars.devtools_markup_gridinspector_opened AS scalar_parent_devtools_markup_gridinspector_opened,
    processes.parent.scalars.devtools_onboarding_is_devtools_user AS scalar_parent_devtools_onboarding_is_devtools_user,
    processes.parent.scalars.devtools_responsive_toolbox_opened_first AS scalar_parent_devtools_responsive_toolbox_opened_first,
    processes.parent.scalars.devtools_rules_flexboxhighlighter_opened AS scalar_parent_devtools_rules_flexboxhighlighter_opened,
    processes.parent.scalars.devtools_rules_gridinspector_opened AS scalar_parent_devtools_rules_gridinspector_opened,
    processes.parent.scalars.devtools_shadowdom_reveal_link_clicked AS scalar_parent_devtools_shadowdom_reveal_link_clicked,
    processes.parent.scalars.devtools_shadowdom_shadow_root_displayed AS scalar_parent_devtools_shadowdom_shadow_root_displayed,
    processes.parent.scalars.devtools_shadowdom_shadow_root_expanded AS scalar_parent_devtools_shadowdom_shadow_root_expanded,
    processes.parent.scalars.devtools_toolbar_eyedropper_opened AS scalar_parent_devtools_toolbar_eyedropper_opened,
    processes.parent.scalars.devtools_webreplay_load_recording AS scalar_parent_devtools_webreplay_load_recording,
    processes.parent.scalars.devtools_webreplay_new_recording AS scalar_parent_devtools_webreplay_new_recording,
    processes.parent.scalars.devtools_webreplay_reload_recording AS scalar_parent_devtools_webreplay_reload_recording,
    processes.parent.scalars.devtools_webreplay_save_recording AS scalar_parent_devtools_webreplay_save_recording,
    processes.parent.scalars.devtools_webreplay_stop_recording AS scalar_parent_devtools_webreplay_stop_recording,
    processes.parent.scalars.dom_contentprocess_build_id_mismatch AS scalar_parent_dom_contentprocess_buildID_mismatch,
    processes.parent.scalars.dom_contentprocess_os_priority_change_considered AS scalar_parent_dom_contentprocess_os_priority_change_considered,
    processes.parent.scalars.dom_contentprocess_os_priority_lowered AS scalar_parent_dom_contentprocess_os_priority_lowered,
    processes.parent.scalars.dom_contentprocess_os_priority_raised AS scalar_parent_dom_contentprocess_os_priority_raised,
    processes.parent.scalars.dom_contentprocess_troubled_due_to_memory AS scalar_parent_dom_contentprocess_troubled_due_to_memory,
    processes.parent.scalars.dom_parentprocess_private_window_used AS scalar_parent_dom_parentprocess_private_window_used,
    processes.parent.scalars.encoding_override_used AS scalar_parent_encoding_override_used,
    processes.parent.scalars.first_startup_elapsed AS scalar_parent_firstStartup_elapsed,
    processes.parent.scalars.first_startup_status_code AS scalar_parent_firstStartup_statusCode,
    processes.parent.scalars.formautofill_addresses_fill_type_autofill AS scalar_parent_formautofill_addresses_fill_type_autofill,
    processes.parent.scalars.formautofill_addresses_fill_type_autofill_update AS scalar_parent_formautofill_addresses_fill_type_autofill_update,
    processes.parent.scalars.formautofill_addresses_fill_type_manual AS scalar_parent_formautofill_addresses_fill_type_manual,
    processes.parent.scalars.formautofill_availability AS scalar_parent_formautofill_availability,
    processes.parent.scalars.formautofill_credit_cards_fill_type_autofill AS scalar_parent_formautofill_creditCards_fill_type_autofill,
    processes.parent.scalars.formautofill_credit_cards_fill_type_autofill_modified AS scalar_parent_formautofill_creditCards_fill_type_autofill_modified,
    processes.parent.scalars.formautofill_credit_cards_fill_type_manual AS scalar_parent_formautofill_creditCards_fill_type_manual,
    processes.parent.scalars.gfx_hdr_windows_display_colorspace_bitfield AS scalar_parent_gfx_hdr_windows_display_colorspace_bitfield,
    processes.parent.scalars.idb_failure_fileinfo_error AS scalar_parent_idb_failure_fileinfo_error,
    processes.parent.scalars.idb_type_persistent_count AS scalar_parent_idb_type_persistent_count,
    processes.parent.scalars.idb_type_temporary_count AS scalar_parent_idb_type_temporary_count,
    processes.parent.scalars.identity_fxaccounts_missed_commands_fetched AS scalar_parent_identity_fxaccounts_missed_commands_fetched,
    processes.parent.scalars.images_webp_content_observed AS scalar_parent_images_webp_content_observed,
    processes.parent.scalars.images_webp_probe_observed AS scalar_parent_images_webp_probe_observed,
    processes.parent.scalars.media_allowed_autoplay_no_audio_track_count AS scalar_parent_media_allowed_autoplay_no_audio_track_count,
    processes.parent.scalars.media_autoplay_default_blocked AS scalar_parent_media_autoplay_default_blocked,
    processes.parent.scalars.media_autoplay_would_be_allowed_count AS scalar_parent_media_autoplay_would_be_allowed_count,
    processes.parent.scalars.media_autoplay_would_not_be_allowed_count AS scalar_parent_media_autoplay_would_not_be_allowed_count,
    NULL AS scalar_parent_media_blocked_autoplay_no_audio_track_count,
    processes.parent.scalars.media_blocked_no_metadata AS scalar_parent_media_blocked_no_metadata,
    processes.parent.scalars.media_blocked_no_metadata_endup_no_audio_track AS scalar_parent_media_blocked_no_metadata_endup_no_audio_track,
    processes.parent.scalars.media_page_count AS scalar_parent_media_page_count,
    processes.parent.scalars.media_page_had_media_count AS scalar_parent_media_page_had_media_count,
    processes.parent.scalars.media_page_had_play_revoked_count AS scalar_parent_media_page_had_play_revoked_count,
    processes.parent.scalars.mediarecorder_recording_count AS scalar_parent_mediarecorder_recording_count,
    processes.parent.scalars.navigator_storage_estimate_count AS scalar_parent_navigator_storage_estimate_count,
    processes.parent.scalars.navigator_storage_persist_count AS scalar_parent_navigator_storage_persist_count,
    processes.parent.scalars.networking_data_transferred_captive_portal AS scalar_parent_networking_data_transferred_captive_portal,
    processes.parent.scalars.networking_ftp_opened_channels_files AS scalar_parent_networking_ftp_opened_channels_files,
    processes.parent.scalars.networking_ftp_opened_channels_listings AS scalar_parent_networking_ftp_opened_channels_listings,
    processes.parent.scalars.networking_http_connections_captive_portal AS scalar_parent_networking_http_connections_captive_portal,
    processes.parent.scalars.networking_http_transactions_captive_portal AS scalar_parent_networking_http_transactions_captive_portal,
    processes.parent.scalars.os_environment_is_admin_without_uac AS scalar_parent_os_environment_is_admin_without_uac,
    processes.parent.scalars.pdf_viewer_fallback_shown AS scalar_parent_pdf_viewer_fallback_shown,
    processes.parent.scalars.pdf_viewer_print AS scalar_parent_pdf_viewer_print,
    processes.parent.scalars.pdf_viewer_used AS scalar_parent_pdf_viewer_used,
    processes.parent.scalars.preferences_created_new_user_prefs_file AS scalar_parent_preferences_created_new_user_prefs_file,
    processes.parent.scalars.preferences_prefs_file_was_invalid AS scalar_parent_preferences_prefs_file_was_invalid,
    processes.parent.scalars.preferences_prevent_accessibility_services AS scalar_parent_preferences_prevent_accessibility_services,
    processes.parent.scalars.preferences_read_user_js AS scalar_parent_preferences_read_user_js,
    processes.parent.scalars.pwmgr_potentially_breached_passwords AS scalar_parent_pwmgr_potentially_breached_passwords,
    processes.parent.scalars.screenshots_copy AS scalar_parent_screenshots_copy,
    processes.parent.scalars.screenshots_download AS scalar_parent_screenshots_download,
    processes.parent.scalars.screenshots_upload AS scalar_parent_screenshots_upload,
    processes.parent.scalars.script_preloader_mainthread_recompile AS scalar_parent_script_preloader_mainthread_recompile,
    processes.parent.scalars.security_intermediate_preloading_num_pending AS scalar_parent_security_intermediate_preloading_num_pending,
    processes.parent.scalars.security_intermediate_preloading_num_preloaded AS scalar_parent_security_intermediate_preloading_num_preloaded,
    processes.parent.scalars.security_tls_delegated_credentials_txn AS scalar_parent_security_tls_delegated_credentials_txn,
    processes.parent.scalars.services_sync_fxa_verification_method AS scalar_parent_services_sync_fxa_verification_method,
    processes.parent.scalars.startup_is_cold AS scalar_parent_startup_is_cold,
    processes.parent.scalars.startup_profile_selection_reason AS scalar_parent_startup_profile_selection_reason,
    processes.parent.scalars.storage_sync_api_usage_extensions_using AS scalar_parent_storage_sync_api_usage_extensions_using,
    processes.parent.scalars.sw_alternative_body_used_count AS scalar_parent_sw_alternative_body_used_count,
    processes.parent.scalars.sw_cors_res_for_so_req_count AS scalar_parent_sw_cors_res_for_so_req_count,
    processes.parent.scalars.sw_synthesized_res_count AS scalar_parent_sw_synthesized_res_count,
    processes.parent.scalars.telemetry_about_telemetry_pageload AS scalar_parent_telemetry_about_telemetry_pageload,
    processes.parent.scalars.telemetry_data_upload_optin AS scalar_parent_telemetry_data_upload_optin,
    processes.parent.scalars.telemetry_ecosystem_new_send_time AS scalar_parent_telemetry_ecosystem_new_send_time,
    processes.parent.scalars.telemetry_ecosystem_old_send_time AS scalar_parent_telemetry_ecosystem_old_send_time,
    processes.parent.scalars.telemetry_os_shutting_down AS scalar_parent_telemetry_os_shutting_down,
    processes.parent.scalars.telemetry_pending_operations_highwatermark_reached AS scalar_parent_telemetry_pending_operations_highwatermark_reached,
    processes.parent.scalars.telemetry_persistence_timer_hit_count AS scalar_parent_telemetry_persistence_timer_hit_count,
    processes.parent.scalars.telemetry_process_creation_timestamp_inconsistent AS scalar_parent_telemetry_process_creation_timestamp_inconsistent,
    processes.parent.scalars.telemetry_profile_directory_scan_date AS scalar_parent_telemetry_profile_directory_scan_date,
    processes.parent.scalars.telemetry_profile_directory_scans AS scalar_parent_telemetry_profile_directory_scans,
    processes.parent.scalars.timestamps_about_home_topsites_first_paint AS scalar_parent_timestamps_about_home_topsites_first_paint,
    processes.parent.scalars.timestamps_first_paint AS scalar_parent_timestamps_first_paint,
    processes.parent.scalars.update_fix_permissions_attempted AS scalar_parent_update_fix_permissions_attempted,
    processes.parent.scalars.update_session_downloads_bits_complete_bytes AS scalar_parent_update_session_downloads_bits_complete_bytes,
    processes.parent.scalars.update_session_downloads_bits_complete_seconds AS scalar_parent_update_session_downloads_bits_complete_seconds,
    processes.parent.scalars.update_session_downloads_bits_partial_bytes AS scalar_parent_update_session_downloads_bits_partial_bytes,
    processes.parent.scalars.update_session_downloads_bits_partial_seconds AS scalar_parent_update_session_downloads_bits_partial_seconds,
    processes.parent.scalars.update_session_downloads_internal_complete_bytes AS scalar_parent_update_session_downloads_internal_complete_bytes,
    processes.parent.scalars.update_session_downloads_internal_complete_seconds AS scalar_parent_update_session_downloads_internal_complete_seconds,
    processes.parent.scalars.update_session_downloads_internal_partial_bytes AS scalar_parent_update_session_downloads_internal_partial_bytes,
    processes.parent.scalars.update_session_downloads_internal_partial_seconds AS scalar_parent_update_session_downloads_internal_partial_seconds,
    processes.parent.scalars.update_session_from_app_version AS scalar_parent_update_session_from_app_version,
    processes.parent.scalars.update_session_intervals_apply_complete AS scalar_parent_update_session_intervals_apply_complete,
    processes.parent.scalars.update_session_intervals_apply_partial AS scalar_parent_update_session_intervals_apply_partial,
    processes.parent.scalars.update_session_intervals_check AS scalar_parent_update_session_intervals_check,
    processes.parent.scalars.update_session_intervals_download_bits_complete AS scalar_parent_update_session_intervals_download_bits_complete,
    processes.parent.scalars.update_session_intervals_download_bits_partial AS scalar_parent_update_session_intervals_download_bits_partial,
    processes.parent.scalars.update_session_intervals_download_internal_complete AS scalar_parent_update_session_intervals_download_internal_complete,
    processes.parent.scalars.update_session_intervals_download_internal_partial AS scalar_parent_update_session_intervals_download_internal_partial,
    processes.parent.scalars.update_session_intervals_stage_complete AS scalar_parent_update_session_intervals_stage_complete,
    processes.parent.scalars.update_session_intervals_stage_partial AS scalar_parent_update_session_intervals_stage_partial,
    processes.parent.scalars.update_session_mar_complete_size_bytes AS scalar_parent_update_session_mar_complete_size_bytes,
    processes.parent.scalars.update_session_mar_partial_size_bytes AS scalar_parent_update_session_mar_partial_size_bytes,
    processes.parent.scalars.update_startup_downloads_bits_complete_bytes AS scalar_parent_update_startup_downloads_bits_complete_bytes,
    processes.parent.scalars.update_startup_downloads_bits_complete_seconds AS scalar_parent_update_startup_downloads_bits_complete_seconds,
    processes.parent.scalars.update_startup_downloads_bits_partial_bytes AS scalar_parent_update_startup_downloads_bits_partial_bytes,
    processes.parent.scalars.update_startup_downloads_bits_partial_seconds AS scalar_parent_update_startup_downloads_bits_partial_seconds,
    processes.parent.scalars.update_startup_downloads_internal_complete_bytes AS scalar_parent_update_startup_downloads_internal_complete_bytes,
    processes.parent.scalars.update_startup_downloads_internal_complete_seconds AS scalar_parent_update_startup_downloads_internal_complete_seconds,
    processes.parent.scalars.update_startup_downloads_internal_partial_bytes AS scalar_parent_update_startup_downloads_internal_partial_bytes,
    processes.parent.scalars.update_startup_downloads_internal_partial_seconds AS scalar_parent_update_startup_downloads_internal_partial_seconds,
    processes.parent.scalars.update_startup_from_app_version AS scalar_parent_update_startup_from_app_version,
    processes.parent.scalars.update_startup_intervals_apply_complete AS scalar_parent_update_startup_intervals_apply_complete,
    processes.parent.scalars.update_startup_intervals_apply_partial AS scalar_parent_update_startup_intervals_apply_partial,
    processes.parent.scalars.update_startup_intervals_check AS scalar_parent_update_startup_intervals_check,
    processes.parent.scalars.update_startup_intervals_download_bits_complete AS scalar_parent_update_startup_intervals_download_bits_complete,
    processes.parent.scalars.update_startup_intervals_download_bits_partial AS scalar_parent_update_startup_intervals_download_bits_partial,
    processes.parent.scalars.update_startup_intervals_download_internal_complete AS scalar_parent_update_startup_intervals_download_internal_complete,
    processes.parent.scalars.update_startup_intervals_download_internal_partial AS scalar_parent_update_startup_intervals_download_internal_partial,
    processes.parent.scalars.update_startup_intervals_stage_complete AS scalar_parent_update_startup_intervals_stage_complete,
    processes.parent.scalars.update_startup_intervals_stage_partial AS scalar_parent_update_startup_intervals_stage_partial,
    processes.parent.scalars.update_startup_mar_complete_size_bytes AS scalar_parent_update_startup_mar_complete_size_bytes,
    processes.parent.scalars.update_startup_mar_partial_size_bytes AS scalar_parent_update_startup_mar_partial_size_bytes,
    processes.parent.scalars.webrtc_hostnameobfuscation_disabled_failed AS scalar_parent_webrtc_hostnameobfuscation_disabled_failed,
    processes.parent.scalars.webrtc_hostnameobfuscation_disabled_succeeded AS scalar_parent_webrtc_hostnameobfuscation_disabled_succeeded,
    processes.parent.scalars.webrtc_hostnameobfuscation_enabled_failed AS scalar_parent_webrtc_hostnameobfuscation_enabled_failed,
    processes.parent.scalars.webrtc_hostnameobfuscation_enabled_succeeded AS scalar_parent_webrtc_hostnameobfuscation_enabled_succeeded,
    processes.parent.scalars.webrtc_nicer_stun_retransmits AS scalar_parent_webrtc_nicer_stun_retransmits,
    processes.parent.scalars.webrtc_nicer_turn_401s AS scalar_parent_webrtc_nicer_turn_401s,
    processes.parent.scalars.webrtc_nicer_turn_403s AS scalar_parent_webrtc_nicer_turn_403s,
    processes.parent.scalars.webrtc_nicer_turn_438s AS scalar_parent_webrtc_nicer_turn_438s,
    processes.parent.scalars.webrtc_peerconnection_connected AS scalar_parent_webrtc_peerconnection_connected,
    processes.parent.scalars.webrtc_peerconnection_datachannel_created AS scalar_parent_webrtc_peerconnection_datachannel_created,
    processes.parent.scalars.webrtc_peerconnection_datachannel_max_life_used AS scalar_parent_webrtc_peerconnection_datachannel_max_life_used,
    processes.parent.scalars.webrtc_peerconnection_datachannel_max_retx_and_life_used AS scalar_parent_webrtc_peerconnection_datachannel_max_retx_and_life_used,
    processes.parent.scalars.webrtc_peerconnection_datachannel_max_retx_used AS scalar_parent_webrtc_peerconnection_datachannel_max_retx_used,
    processes.parent.scalars.webrtc_peerconnection_legacy_callback_stats_used AS scalar_parent_webrtc_peerconnection_legacy_callback_stats_used,
    processes.parent.scalars.webrtc_peerconnection_promise_and_callback_stats_used AS scalar_parent_webrtc_peerconnection_promise_and_callback_stats_used,
    processes.parent.scalars.webrtc_peerconnection_promise_stats_used AS scalar_parent_webrtc_peerconnection_promise_stats_used,
    processes.content.scalars.browser_feeds_preview_loaded AS scalar_content_browser_feeds_preview_loaded,
    processes.content.scalars.browser_usage_graphite AS scalar_content_browser_usage_graphite,
    processes.content.scalars.browser_usage_plugin_instantiated AS scalar_content_browser_usage_plugin_instantiated,
    processes.content.scalars.encoding_override_used AS scalar_content_encoding_override_used,
    processes.content.scalars.gfx_omtp_paint_wait_ratio AS scalar_content_gfx_omtp_paint_wait_ratio,
    processes.content.scalars.idb_type_persistent_count AS scalar_content_idb_type_persistent_count,
    processes.content.scalars.idb_type_temporary_count AS scalar_content_idb_type_temporary_count,
    processes.content.scalars.images_webp_content_observed AS scalar_content_images_webp_content_observed,
    processes.content.scalars.images_webp_probe_observed AS scalar_content_images_webp_probe_observed,
    processes.content.scalars.mathml_doc_count AS scalar_content_mathml_doc_count,
    processes.content.scalars.media_allowed_autoplay_no_audio_track_count AS scalar_content_media_allowed_autoplay_no_audio_track_count,
    processes.content.scalars.media_autoplay_default_blocked AS scalar_content_media_autoplay_default_blocked,
    processes.content.scalars.media_autoplay_would_be_allowed_count AS scalar_content_media_autoplay_would_be_allowed_count,
    processes.content.scalars.media_autoplay_would_not_be_allowed_count AS scalar_content_media_autoplay_would_not_be_allowed_count,
    NULL AS scalar_content_media_blocked_autoplay_no_audio_track_count,
    processes.content.scalars.media_blocked_no_metadata AS scalar_content_media_blocked_no_metadata,
    processes.content.scalars.media_blocked_no_metadata_endup_no_audio_track AS scalar_content_media_blocked_no_metadata_endup_no_audio_track,
    processes.content.scalars.media_page_count AS scalar_content_media_page_count,
    processes.content.scalars.media_page_had_media_count AS scalar_content_media_page_had_media_count,
    processes.content.scalars.media_page_had_play_revoked_count AS scalar_content_media_page_had_play_revoked_count,
    processes.content.scalars.mediarecorder_recording_count AS scalar_content_mediarecorder_recording_count,
    processes.content.scalars.memoryreporter_max_ghost_windows AS scalar_content_memoryreporter_max_ghost_windows,
    processes.content.scalars.navigator_storage_estimate_count AS scalar_content_navigator_storage_estimate_count,
    processes.content.scalars.navigator_storage_persist_count AS scalar_content_navigator_storage_persist_count,
    processes.content.scalars.pdf_viewer_fallback_shown AS scalar_content_pdf_viewer_fallback_shown,
    processes.content.scalars.pdf_viewer_print AS scalar_content_pdf_viewer_print,
    processes.content.scalars.pdf_viewer_used AS scalar_content_pdf_viewer_used,
    processes.content.scalars.script_preloader_mainthread_recompile AS scalar_content_script_preloader_mainthread_recompile,
    processes.content.scalars.sw_alternative_body_used_count AS scalar_content_sw_alternative_body_used_count,
    processes.content.scalars.sw_cors_res_for_so_req_count AS scalar_content_sw_cors_res_for_so_req_count,
    processes.content.scalars.sw_synthesized_res_count AS scalar_content_sw_synthesized_res_count,
    processes.content.scalars.telemetry_discarded_accumulations AS scalar_content_telemetry_discarded_accumulations,
    processes.content.scalars.telemetry_discarded_child_events AS scalar_content_telemetry_discarded_child_events,
    processes.content.scalars.telemetry_discarded_keyed_accumulations AS scalar_content_telemetry_discarded_keyed_accumulations,
    processes.content.scalars.telemetry_discarded_keyed_scalar_actions AS scalar_content_telemetry_discarded_keyed_scalar_actions,
    processes.content.scalars.telemetry_discarded_scalar_actions AS scalar_content_telemetry_discarded_scalar_actions,
    processes.content.scalars.telemetry_process_creation_timestamp_inconsistent AS scalar_content_telemetry_process_creation_timestamp_inconsistent,
    processes.content.scalars.telemetry_profile_directory_scans AS scalar_content_telemetry_profile_directory_scans,
    processes.content.scalars.webrtc_hostnameobfuscation_disabled_failed AS scalar_content_webrtc_hostnameobfuscation_disabled_failed,
    processes.content.scalars.webrtc_hostnameobfuscation_disabled_succeeded AS scalar_content_webrtc_hostnameobfuscation_disabled_succeeded,
    processes.content.scalars.webrtc_hostnameobfuscation_enabled_failed AS scalar_content_webrtc_hostnameobfuscation_enabled_failed,
    processes.content.scalars.webrtc_hostnameobfuscation_enabled_succeeded AS scalar_content_webrtc_hostnameobfuscation_enabled_succeeded,
    processes.content.scalars.webrtc_nicer_stun_retransmits AS scalar_content_webrtc_nicer_stun_retransmits,
    processes.content.scalars.webrtc_nicer_turn_401s AS scalar_content_webrtc_nicer_turn_401s,
    processes.content.scalars.webrtc_nicer_turn_403s AS scalar_content_webrtc_nicer_turn_403s,
    processes.content.scalars.webrtc_nicer_turn_438s AS scalar_content_webrtc_nicer_turn_438s,
    processes.content.scalars.webrtc_peerconnection_connected AS scalar_content_webrtc_peerconnection_connected,
    processes.content.scalars.webrtc_peerconnection_datachannel_created AS scalar_content_webrtc_peerconnection_datachannel_created,
    processes.content.scalars.webrtc_peerconnection_datachannel_max_life_used AS scalar_content_webrtc_peerconnection_datachannel_max_life_used,
    processes.content.scalars.webrtc_peerconnection_datachannel_max_retx_and_life_used AS scalar_content_webrtc_peerconnection_datachannel_max_retx_and_life_used,
    processes.content.scalars.webrtc_peerconnection_datachannel_max_retx_used AS scalar_content_webrtc_peerconnection_datachannel_max_retx_used,
    processes.content.scalars.webrtc_peerconnection_legacy_callback_stats_used AS scalar_content_webrtc_peerconnection_legacy_callback_stats_used,
    processes.content.scalars.webrtc_peerconnection_promise_and_callback_stats_used AS scalar_content_webrtc_peerconnection_promise_and_callback_stats_used,
    processes.content.scalars.webrtc_peerconnection_promise_stats_used AS scalar_content_webrtc_peerconnection_promise_stats_used,
    processes.gpu.scalars.browser_usage_graphite AS scalar_gpu_browser_usage_graphite,
    processes.gpu.scalars.telemetry_discarded_accumulations AS scalar_gpu_telemetry_discarded_accumulations,
    processes.gpu.scalars.telemetry_discarded_child_events AS scalar_gpu_telemetry_discarded_child_events,
    processes.gpu.scalars.telemetry_discarded_keyed_accumulations AS scalar_gpu_telemetry_discarded_keyed_accumulations,
    processes.gpu.scalars.telemetry_discarded_keyed_scalar_actions AS scalar_gpu_telemetry_discarded_keyed_scalar_actions,
    processes.gpu.scalars.telemetry_discarded_scalar_actions AS scalar_gpu_telemetry_discarded_scalar_actions,
    processes.gpu.scalars.telemetry_profile_directory_scans AS scalar_gpu_telemetry_profile_directory_scans,
    processes.parent.keyed_scalars.a11y_theme AS scalar_parent_a11y_theme,
    processes.parent.keyed_scalars.browser_engagement_navigation_about_home AS scalar_parent_browser_engagement_navigation_about_home,
    processes.parent.keyed_scalars.browser_engagement_navigation_about_newtab AS scalar_parent_browser_engagement_navigation_about_newtab,
    processes.parent.keyed_scalars.browser_engagement_navigation_contextmenu AS scalar_parent_browser_engagement_navigation_contextmenu,
    processes.parent.keyed_scalars.browser_engagement_navigation_searchbar AS scalar_parent_browser_engagement_navigation_searchbar,
    processes.parent.keyed_scalars.browser_engagement_navigation_urlbar AS scalar_parent_browser_engagement_navigation_urlbar,
    processes.parent.keyed_scalars.browser_engagement_navigation_webextension AS scalar_parent_browser_engagement_navigation_webextension,
    processes.parent.keyed_scalars.browser_errors_collected_count_by_filename AS scalar_parent_browser_errors_collected_count_by_filename,
    processes.parent.keyed_scalars.browser_search_ad_clicks AS scalar_parent_browser_search_ad_clicks,
    processes.parent.keyed_scalars.browser_search_with_ads AS scalar_parent_browser_search_with_ads,
    processes.parent.keyed_scalars.devtools_accessibility_accessible_context_menu_item_activated AS scalar_parent_devtools_accessibility_accessible_context_menu_item_activated,
    processes.parent.keyed_scalars.devtools_accessibility_audit_activated AS scalar_parent_devtools_accessibility_audit_activated,
    processes.parent.keyed_scalars.devtools_accessibility_select_accessible_for_node AS scalar_parent_devtools_accessibility_select_accessible_for_node,
    processes.parent.keyed_scalars.devtools_accessibility_simulation_activated AS scalar_parent_devtools_accessibility_simulation_activated,
    processes.parent.keyed_scalars.devtools_current_theme AS scalar_parent_devtools_current_theme,
    processes.parent.keyed_scalars.devtools_inspector_three_pane_enabled AS scalar_parent_devtools_inspector_three_pane_enabled,
    processes.parent.keyed_scalars.devtools_responsive_open_trigger AS scalar_parent_devtools_responsive_open_trigger,
    processes.parent.keyed_scalars.devtools_tool_registered AS scalar_parent_devtools_tool_registered,
    processes.parent.keyed_scalars.devtools_toolbox_tabs_reordered AS scalar_parent_devtools_toolbox_tabs_reordered,
    processes.parent.keyed_scalars.devtools_tooltip_shown AS scalar_parent_devtools_tooltip_shown,
    processes.parent.keyed_scalars.extensions_updates_rdf AS scalar_parent_extensions_updates_rdf,
    processes.parent.keyed_scalars.gfx_advanced_layers_failure_id AS scalar_parent_gfx_advanced_layers_failure_id,
    processes.parent.keyed_scalars.images_webp_content_frequency AS scalar_parent_images_webp_content_frequency,
    ARRAY<STRUCT<key STRING, value INT64>>[] AS scalar_parent_networking_data_transferred,
    processes.parent.keyed_scalars.networking_data_transferred_kb AS scalar_parent_networking_data_transferred_kb,
    processes.parent.keyed_scalars.networking_data_transferred_v3_kb AS scalar_parent_networking_data_transferred_v3_kb,
    processes.parent.keyed_scalars.normandy_recipe_freshness AS scalar_parent_normandy_recipe_freshness,
    processes.parent.keyed_scalars.pictureinpicture_closed_method AS scalar_parent_pictureinpicture_closed_method,
    processes.parent.keyed_scalars.preferences_browser_home_page_change AS scalar_parent_preferences_browser_home_page_change,
    processes.parent.keyed_scalars.preferences_browser_home_page_count AS scalar_parent_preferences_browser_home_page_count,
    processes.parent.keyed_scalars.preferences_search_query AS scalar_parent_preferences_search_query,
    processes.parent.keyed_scalars.preferences_use_bookmark AS scalar_parent_preferences_use_bookmark,
    processes.parent.keyed_scalars.preferences_use_current_page AS scalar_parent_preferences_use_current_page,
    processes.parent.keyed_scalars.qm_origin_directory_unexpected_filename AS scalar_parent_qm_origin_directory_unexpected_filename,
    processes.parent.keyed_scalars.resistfingerprinting_content_window_size AS scalar_parent_resistfingerprinting_content_window_size,
    processes.parent.keyed_scalars.sandbox_no_job AS scalar_parent_sandbox_no_job,
    processes.parent.keyed_scalars.security_client_cert AS scalar_parent_security_client_cert,
    processes.parent.keyed_scalars.security_contentblocker_permissions AS scalar_parent_security_contentblocker_permissions,
    processes.parent.keyed_scalars.security_pkcs11_modules_loaded AS scalar_parent_security_pkcs11_modules_loaded,
    processes.parent.keyed_scalars.security_webauthn_used AS scalar_parent_security_webauthn_used,
    processes.parent.keyed_scalars.services_sync_sync_login_state_transitions AS scalar_parent_services_sync_sync_login_state_transitions,
    processes.parent.keyed_scalars.storage_sync_api_usage_items_stored AS scalar_parent_storage_sync_api_usage_items_stored,
    processes.parent.keyed_scalars.storage_sync_api_usage_storage_consumed AS scalar_parent_storage_sync_api_usage_storage_consumed,
    processes.parent.keyed_scalars.telemetry_accumulate_clamped_values AS scalar_parent_telemetry_accumulate_clamped_values,
    processes.parent.keyed_scalars.telemetry_accumulate_unknown_histogram_keys AS scalar_parent_telemetry_accumulate_unknown_histogram_keys,
    processes.parent.keyed_scalars.telemetry_event_counts AS scalar_parent_telemetry_event_counts,
    processes.parent.keyed_scalars.telemetry_keyed_scalars_exceed_limit AS scalar_parent_telemetry_keyed_scalars_exceed_limit,
    processes.parent.keyed_scalars.update_binarytransparencyresult AS scalar_parent_update_binarytransparencyresult,
    processes.parent.keyed_scalars.update_bitshresult AS scalar_parent_update_bitshresult,
    processes.parent.keyed_scalars.urlbar_searchmode_bookmarkmenu AS scalar_parent_urlbar_searchmode_bookmarkmenu,
    processes.parent.keyed_scalars.urlbar_searchmode_handoff AS scalar_parent_urlbar_searchmode_handoff,
    processes.parent.keyed_scalars.urlbar_searchmode_keywordoffer AS scalar_parent_urlbar_searchmode_keywordoffer,
    processes.parent.keyed_scalars.urlbar_searchmode_oneoff AS scalar_parent_urlbar_searchmode_oneoff,
    processes.parent.keyed_scalars.urlbar_searchmode_other AS scalar_parent_urlbar_searchmode_other,
    processes.parent.keyed_scalars.urlbar_searchmode_shortcut AS scalar_parent_urlbar_searchmode_shortcut,
    processes.parent.keyed_scalars.urlbar_searchmode_tabmenu AS scalar_parent_urlbar_searchmode_tabmenu,
    processes.parent.keyed_scalars.urlbar_searchmode_tabtosearch AS scalar_parent_urlbar_searchmode_tabtosearch,
    processes.parent.keyed_scalars.urlbar_searchmode_tabtosearch_onboard AS scalar_parent_urlbar_searchmode_tabtosearch_onboard,
    processes.parent.keyed_scalars.urlbar_searchmode_topsites_newtab AS scalar_parent_urlbar_searchmode_topsites_newtab,
    processes.parent.keyed_scalars.urlbar_searchmode_topsites_urlbar AS scalar_parent_urlbar_searchmode_topsites_urlbar,
    processes.parent.keyed_scalars.urlbar_searchmode_touchbar AS scalar_parent_urlbar_searchmode_touchbar,
    processes.parent.keyed_scalars.urlbar_searchmode_typed AS scalar_parent_urlbar_searchmode_typed,
    processes.parent.keyed_scalars.widget_ime_name_on_linux AS scalar_parent_widget_ime_name_on_linux,
    processes.parent.keyed_scalars.widget_ime_name_on_mac AS scalar_parent_widget_ime_name_on_mac,
    processes.parent.keyed_scalars.widget_ime_name_on_windows AS scalar_parent_widget_ime_name_on_windows,
    processes.content.keyed_scalars.dom_event_confluence_load_count AS scalar_content_dom_event_confluence_load_count,
    processes.content.keyed_scalars.dom_event_office_online_load_count AS scalar_content_dom_event_office_online_load_count,
    processes.content.keyed_scalars.gfx_small_paint_phase_weight AS scalar_content_gfx_small_paint_phase_weight,
    processes.content.keyed_scalars.images_webp_content_frequency AS scalar_content_images_webp_content_frequency,
    processes.content.keyed_scalars.pictureinpicture_opened_method AS scalar_content_pictureinpicture_opened_method,
    processes.content.keyed_scalars.telemetry_accumulate_unknown_histogram_keys AS scalar_content_telemetry_accumulate_unknown_histogram_keys,
    processes.content.keyed_scalars.telemetry_event_counts AS scalar_content_telemetry_event_counts,
    processes.content.keyed_scalars.webrtc_sdp_parser_diff AS scalar_content_webrtc_sdp_parser_diff,
    processes.content.keyed_scalars.webrtc_video_recv_codec_used AS scalar_content_webrtc_video_recv_codec_used,
    processes.content.keyed_scalars.webrtc_video_send_codec_used AS scalar_content_webrtc_video_send_codec_used,
    processes.gpu.keyed_scalars.telemetry_accumulate_unknown_histogram_keys AS scalar_gpu_telemetry_accumulate_unknown_histogram_keys,
    processes.gpu.keyed_scalars.telemetry_event_counts AS scalar_gpu_telemetry_event_counts
  )
);

CREATE TEMP FUNCTION udf_null_if_empty_list(list ANY TYPE) AS (
  IF(ARRAY_LENGTH(list) > 0, list, NULL)
);

WITH histogram_lists AS (
  SELECT
    document_id,
    udf_main_summary_scalars(payload.processes).*,
    ARRAY(
      SELECT AS STRUCT
        udf_null_if_empty_list(mozfun.hist.extract(histogram).values) AS list
      FROM
        UNNEST(
          [
            payload.histograms.a11y_consumers,
            payload.histograms.cert_validation_success_by_ca,
            payload.processes.content.histograms.cert_validation_success_by_ca,
            payload.histograms.checkerboard_severity,
            payload.processes.content.histograms.checkerboard_severity,
            JSON_EXTRACT(
              additional_properties,
              '$.payload.processes.gpu.histograms.CHECKERBOARD_SEVERITY'
            ),
            payload.histograms.composite_time,
            payload.processes.content.histograms.composite_time,
            JSON_EXTRACT(
              additional_properties,
              '$.payload.processes.gpu.histograms.COMPOSITE_TIME'
            ),
            JSON_EXTRACT(
              additional_properties,
              '$.payload.processes.gpu.histograms.CONTENT_FRAME_TIME'
            ),
            payload.histograms.content_paint_time,
            payload.processes.content.histograms.content_paint_time,
            payload.histograms.cookie_behavior,
            payload.histograms.cycle_collector_max_pause,
            payload.processes.content.histograms.cycle_collector_max_pause,
            payload.histograms.devtools_accessibility_picker_time_active_seconds,
            payload.processes.content.histograms.devtools_accessibility_picker_time_active_seconds,
            payload.histograms.devtools_accessibility_service_time_active_seconds,
            payload.processes.content.histograms.devtools_accessibility_service_time_active_seconds,
            payload.histograms.devtools_accessibility_time_active_seconds,
            payload.processes.content.histograms.devtools_accessibility_time_active_seconds,
            payload.histograms.devtools_fonteditor_n_fonts_rendered,
            payload.histograms.devtools_fonteditor_n_font_axes,
            payload.histograms.devtools_toolbox_time_active_seconds,
            payload.processes.content.histograms.devtools_toolbox_time_active_seconds,
            payload.histograms.dns_failed_lookup_time,
            payload.histograms.dns_lookup_time,
            payload.histograms.fx_new_window_ms,
            payload.histograms.fx_page_load_ms_2,
            payload.processes.content.histograms.fx_page_load_ms_2,
            payload.histograms.fx_session_restore_restore_window_ms,
            payload.histograms.fx_session_restore_startup_init_session_ms,
            payload.histograms.fx_session_restore_startup_onload_initial_window_ms,
            payload.histograms.fx_tab_close_time_anim_ms,
            payload.histograms.fx_tab_switch_total_e10s_ms,
            payload.histograms.fx_tab_switch_update_ms,
            payload.histograms.fx_urlbar_selected_result_index,
            payload.processes.content.histograms.fx_urlbar_selected_result_index,
            payload.histograms.fx_urlbar_selected_result_type,
            payload.processes.content.histograms.fx_urlbar_selected_result_type,
            payload.histograms.gc_animation_ms,
            payload.processes.content.histograms.gc_animation_ms,
            payload.histograms.gc_max_pause_ms_2,
            payload.processes.content.histograms.gc_max_pause_ms_2,
            payload.histograms.geolocation_request_granted,
            payload.processes.content.histograms.geolocation_request_granted,
            payload.histograms.ghost_windows,
            payload.processes.content.histograms.ghost_windows,
            payload.histograms.gpu_process_initialization_time_ms,
            payload.processes.content.histograms.gpu_process_initialization_time_ms,
            JSON_EXTRACT(
              additional_properties,
              '$.payload.processes.gpu.histograms.GPU_PROCESS_INITIALIZATION_TIME_MS'
            ),
            payload.histograms.gpu_process_launch_time_ms_2,
            payload.processes.content.histograms.gpu_process_launch_time_ms_2,
            payload.histograms.http_channel_disposition,
            payload.processes.content.histograms.http_channel_disposition,
            payload.histograms.http_pageload_is_ssl,
            payload.processes.content.histograms.http_pageload_is_ssl,
            payload.histograms.http_transaction_is_ssl,
            payload.processes.content.histograms.http_transaction_is_ssl,
            payload.histograms.input_event_response_coalesced_ms,
            payload.processes.content.histograms.input_event_response_coalesced_ms,
            payload.histograms.memory_heap_allocated,
            payload.processes.content.histograms.memory_heap_allocated,
            payload.histograms.memory_resident_fast,
            payload.processes.content.histograms.memory_resident_fast,
            payload.histograms.memory_total,
            payload.histograms.memory_unique,
            payload.processes.content.histograms.memory_unique,
            payload.processes.content.histograms.memory_unique_content_startup,
            payload.histograms.memory_vsize,
            payload.processes.content.histograms.memory_vsize,
            payload.histograms.memory_vsize_max_contiguous,
            payload.processes.content.histograms.memory_vsize_max_contiguous,
            payload.histograms.network_cache_metadata_first_read_time_ms,
            payload.processes.content.histograms.network_cache_metadata_first_read_time_ms,
            payload.histograms.network_cache_v2_hit_time_ms,
            payload.processes.content.histograms.network_cache_v2_hit_time_ms,
            payload.histograms.network_cache_v2_miss_time_ms,
            payload.processes.content.histograms.network_cache_v2_miss_time_ms,
            payload.histograms.places_autocomplete_6_first_results_time_ms,
            payload.histograms.plugin_shutdown_ms,
            payload.processes.content.histograms.plugin_shutdown_ms,
            payload.histograms.pwmgr_blocklist_num_sites,
            payload.histograms.pwmgr_form_autofill_result,
            payload.processes.content.histograms.pwmgr_form_autofill_result,
            payload.histograms.pwmgr_login_last_used_days,
            payload.histograms.pwmgr_login_page_safety,
            payload.processes.content.histograms.pwmgr_login_page_safety,
            payload.histograms.pwmgr_manage_opened,
            payload.histograms.pwmgr_manage_visibility_toggled,
            payload.histograms.pwmgr_num_passwords_per_hostname,
            payload.histograms.pwmgr_num_saved_passwords,
            payload.histograms.pwmgr_prompt_remember_action,
            payload.histograms.pwmgr_prompt_update_action,
            payload.histograms.pwmgr_saving_enabled,
            payload.histograms.search_service_init_ms,
            payload.histograms.ssl_handshake_result,
            payload.processes.content.histograms.ssl_handshake_result,
            payload.histograms.ssl_handshake_version,
            payload.processes.content.histograms.ssl_handshake_version,
            payload.histograms.ssl_tls12_intolerance_reason_pre,
            payload.processes.content.histograms.ssl_tls12_intolerance_reason_pre,
            payload.histograms.ssl_tls13_intolerance_reason_pre,
            payload.processes.content.histograms.ssl_tls13_intolerance_reason_pre,
            payload.processes.content.histograms.time_to_dom_complete_ms,
            payload.processes.content.histograms.time_to_dom_content_loaded_end_ms,
            payload.processes.content.histograms.time_to_dom_content_loaded_start_ms,
            payload.processes.content.histograms.time_to_dom_interactive_ms,
            payload.processes.content.histograms.time_to_dom_loading_ms,
            payload.histograms.time_to_first_click_ms,
            payload.processes.content.histograms.time_to_first_click_ms,
            payload.histograms.time_to_first_interaction_ms,
            payload.processes.content.histograms.time_to_first_interaction_ms,
            payload.histograms.time_to_first_key_input_ms,
            payload.processes.content.histograms.time_to_first_key_input_ms,
            payload.histograms.time_to_first_mouse_move_ms,
            payload.processes.content.histograms.time_to_first_mouse_move_ms,
            payload.histograms.time_to_first_scroll_ms,
            payload.processes.content.histograms.time_to_first_scroll_ms,
            payload.processes.content.histograms.time_to_load_event_end_ms,
            payload.processes.content.histograms.time_to_load_event_start_ms,
            payload.histograms.time_to_non_blank_paint_ms,
            payload.processes.content.histograms.time_to_non_blank_paint_ms,
            payload.histograms.time_to_response_start_ms,
            payload.processes.content.histograms.time_to_response_start_ms,
            payload.histograms.touch_enabled_device,
            payload.histograms.tracking_protection_enabled,
            payload.histograms.update_download_code_partial,
            payload.histograms.update_download_code_complete,
            payload.histograms.update_bits_result_partial,
            payload.histograms.update_bits_result_complete,
            payload.histograms.update_state_code_partial_stage,
            payload.histograms.update_state_code_complete_stage,
            payload.histograms.update_status_error_code_complete_stage,
            payload.histograms.update_status_error_code_partial_stage,
            payload.histograms.update_state_code_partial_startup,
            payload.histograms.update_state_code_complete_startup,
            payload.histograms.update_status_error_code_partial_startup,
            payload.histograms.update_status_error_code_complete_startup,
            payload.histograms.webext_background_page_load_ms,
            payload.histograms.webext_browseraction_popup_open_ms,
            payload.histograms.webext_content_script_injection_ms,
            payload.processes.content.histograms.webext_content_script_injection_ms,
            payload.histograms.webext_extension_startup_ms,
            payload.histograms.webext_pageaction_popup_open_ms,
            payload.histograms.webext_storage_local_get_ms,
            payload.processes.content.histograms.webext_storage_local_get_ms,
            payload.histograms.webext_storage_local_set_ms,
            payload.processes.content.histograms.webext_storage_local_set_ms,
            payload.histograms.webvr_time_spent_viewing_in_2d,
            payload.processes.content.histograms.webvr_time_spent_viewing_in_2d,
            payload.histograms.webvr_time_spent_viewing_in_oculus,
            JSON_EXTRACT(
              additional_properties,
              '$.payload.processes.gpu.histograms.WEBVR_TIME_SPENT_VIEWING_IN_OCULUS'
            ),
            payload.histograms.webvr_time_spent_viewing_in_openvr,
            JSON_EXTRACT(
              additional_properties,
              '$.payload.processes.gpu.histograms.WEBVR_TIME_SPENT_VIEWING_IN_OPENVR'
            ),
            payload.histograms.webvr_users_view_in,
            JSON_EXTRACT(
              additional_properties,
              '$.payload.processes.gpu.histograms.WEBVR_USERS_VIEW_IN'
            ),
            payload.processes.content.histograms.webvr_users_view_in,
            payload.histograms.a11y_instantiated_flag,
            payload.histograms.search_reset_result,
            payload.processes.content.histograms.search_reset_result,
            payload.histograms.gc_max_pause_ms,
            payload.processes.content.histograms.gc_max_pause_ms,
            payload.histograms.content_frame_time,
            payload.processes.content.histograms.tracking_protection_enabled,
            payload.processes.content.histograms.places_autocomplete_6_first_results_time_ms,
            payload.processes.content.histograms.memory_total,
            payload.processes.content.histograms.fx_session_restore_restore_window_ms,
            payload.processes.content.histograms.fx_session_restore_startup_init_session_ms,
            payload.processes.content.histograms.fx_session_restore_startup_onload_initial_window_ms,
            payload.processes.content.histograms.fx_tab_close_time_anim_ms,
            payload.processes.content.histograms.fx_tab_switch_update_ms,
            payload.processes.content.histograms.fx_new_window_ms,
            payload.processes.content.histograms.search_service_init_ms
          ]
        ) AS histogram
    ) AS values_histograms,
    ARRAY(
      SELECT AS STRUCT
        udf_null_if_empty_list(
          ARRAY(
            SELECT AS STRUCT
              key,
              mozfun.hist.extract(value).values AS value
            FROM
              UNNEST(histogram)
          )
        ) AS list
      FROM
        UNNEST(
          [
            -- format:off
            STRUCT(payload.processes.content.keyed_histograms.fx_urlbar_selected_result_index_by_type AS histogram),
            STRUCT(payload.keyed_histograms.fx_urlbar_selected_result_index_by_type AS histogram),
            STRUCT(payload.processes.content.keyed_histograms.ipc_read_main_thread_latency_ms AS histogram),
            STRUCT(payload.processes.gpu.keyed_histograms.ipc_read_main_thread_latency_ms AS histogram),
            STRUCT(payload.keyed_histograms.ipc_read_main_thread_latency_ms AS histogram),
            STRUCT(payload.keyed_histograms.memory_distribution_among_content AS histogram),
            STRUCT(payload.processes.content.keyed_histograms.memory_distribution_among_content AS histogram)
            -- format:on
          ]
        )
    ) AS keyed_values_histograms,
    ARRAY(
      SELECT AS STRUCT
        udf.get_key(mozfun.hist.extract(histogram).values, 0) AS histogram
      FROM
        UNNEST(
          [
            payload.processes.content.histograms.devtools_aboutdebugging_opened_count,
            payload.processes.content.histograms.devtools_animationinspector_opened_count,
            payload.processes.content.histograms.devtools_browserconsole_opened_count,
            payload.processes.content.histograms.devtools_canvasdebugger_opened_count,
            payload.processes.content.histograms.devtools_computedview_opened_count,
            payload.processes.content.histograms.devtools_custom_opened_count,
            payload.processes.content.histograms.devtools_dom_opened_count,
            payload.processes.content.histograms.devtools_eyedropper_opened_count,
            payload.processes.content.histograms.devtools_fontinspector_opened_count,
            payload.processes.content.histograms.devtools_inspector_opened_count,
            payload.processes.content.histograms.devtools_jsbrowserdebugger_opened_count,
            payload.processes.content.histograms.devtools_jsdebugger_opened_count,
            payload.processes.content.histograms.devtools_jsprofiler_opened_count,
            payload.processes.content.histograms.devtools_layoutview_opened_count,
            payload.processes.content.histograms.devtools_memory_opened_count,
            payload.processes.content.histograms.devtools_menu_eyedropper_opened_count,
            payload.processes.content.histograms.devtools_netmonitor_opened_count,
            payload.processes.content.histograms.devtools_options_opened_count,
            payload.processes.content.histograms.devtools_paintflashing_opened_count,
            payload.processes.content.histograms.devtools_picker_eyedropper_opened_count,
            payload.processes.content.histograms.devtools_responsive_opened_count,
            payload.processes.content.histograms.devtools_ruleview_opened_count,
            payload.processes.content.histograms.devtools_scratchpad_opened_count,
            payload.processes.content.histograms.devtools_scratchpad_window_opened_count,
            payload.processes.content.histograms.devtools_shadereditor_opened_count,
            payload.processes.content.histograms.devtools_storage_opened_count,
            payload.processes.content.histograms.devtools_styleeditor_opened_count,
            payload.processes.content.histograms.devtools_webaudioeditor_opened_count,
            payload.processes.content.histograms.devtools_webconsole_opened_count,
            payload.processes.content.histograms.devtools_webide_opened_count,
            payload.processes.content.histograms.number_of_profiles,
            payload.processes.content.histograms.pwmgr_manage_copied_password,
            payload.processes.content.histograms.pwmgr_manage_copied_username,
            payload.processes.content.histograms.pwmgr_manage_deleted,
            payload.histograms.devtools_aboutdebugging_opened_count,
            payload.histograms.devtools_animationinspector_opened_count,
            payload.histograms.devtools_browserconsole_opened_count,
            payload.histograms.devtools_canvasdebugger_opened_count,
            payload.histograms.devtools_computedview_opened_count,
            payload.histograms.devtools_custom_opened_count,
            payload.histograms.devtools_dom_opened_count,
            payload.histograms.devtools_eyedropper_opened_count,
            payload.histograms.devtools_fontinspector_opened_count,
            payload.histograms.devtools_inspector_opened_count,
            payload.histograms.devtools_jsbrowserdebugger_opened_count,
            payload.histograms.devtools_jsdebugger_opened_count,
            payload.histograms.devtools_jsprofiler_opened_count,
            payload.histograms.devtools_layoutview_opened_count,
            payload.histograms.devtools_memory_opened_count,
            payload.histograms.devtools_menu_eyedropper_opened_count,
            payload.histograms.devtools_netmonitor_opened_count,
            payload.histograms.devtools_options_opened_count,
            payload.histograms.devtools_paintflashing_opened_count,
            payload.histograms.devtools_picker_eyedropper_opened_count,
            payload.histograms.devtools_responsive_opened_count,
            payload.histograms.devtools_ruleview_opened_count,
            payload.histograms.devtools_scratchpad_opened_count,
            payload.histograms.devtools_scratchpad_window_opened_count,
            payload.histograms.devtools_shadereditor_opened_count,
            payload.histograms.devtools_storage_opened_count,
            payload.histograms.devtools_styleeditor_opened_count,
            payload.histograms.devtools_webaudioeditor_opened_count,
            payload.histograms.devtools_webconsole_opened_count,
            payload.histograms.devtools_webide_opened_count,
            payload.histograms.number_of_profiles,
            payload.histograms.pwmgr_manage_copied_password,
            payload.histograms.pwmgr_manage_copied_username,
            payload.histograms.pwmgr_manage_deleted,
            payload.histograms.devtools_developertoolbar_opened_count,
            payload.processes.content.histograms.devtools_developertoolbar_opened_count
          ]
        ) AS histogram
    ) AS count_histograms,
    ARRAY(
      SELECT AS STRUCT
        udf_null_if_empty_list(
          ARRAY(
            SELECT AS STRUCT
              key,
              SAFE_CAST(mozfun.map.get_key(mozfun.hist.extract(value).values, 0) AS INT64) AS value
            FROM
              UNNEST(histogram)
          )
        ) AS list
      FROM
        UNNEST(
          [
            -- format:off
            STRUCT(payload.processes.content.keyed_histograms.sandbox_rejected_syscalls AS histogram),
            STRUCT(payload.keyed_histograms.sandbox_rejected_syscalls AS histogram)
            -- format:on
          ]
        )
    ) AS keyed_count_histograms,
    ARRAY(
      SELECT AS STRUCT
        udf_null_if_empty_list(
          ARRAY(
            SELECT AS STRUCT
              IFNULL(labels[SAFE_OFFSET(key)], 'spill') AS key,
              value
            FROM
              UNNEST(mozfun.hist.extract(histogram).values)
          )
        ) AS list
      FROM
        UNNEST(
          [
            STRUCT(
              [
                payload.processes.content.histograms.fx_searchbar_selected_result_method,
                payload.histograms.fx_searchbar_selected_result_method
              ] AS histograms,
              ['enter', 'enterSelection', 'click'] AS labels
            ),
            (
              [
                payload.histograms.devtools_about_devtools_opened_key,
                payload.histograms.devtools_about_devtools_opened_reason
              ],
              []
            ),
            (
              [payload.histograms.devtools_entry_point],
              [
                'KeyShortcut',
                'SystemMenu',
                'HamburgerMenu',
                'ContextMenu',
                'CommandLine',
                'SessionRestore'
              ]
            ),
            (
              [payload.histograms.devtools_fonteditor_font_type_displayed],
              ['variable', 'nonvariable']
            ),
            (
              [
                payload.processes.content.histograms.fx_urlbar_selected_result_method,
                payload.histograms.fx_urlbar_selected_result_method
              ],
              [
                'enter',
                'enterSelection',
                'click',
                'arrowEnterSelection',
                'tabEnterSelection',
                'rightClickEnter'
              ]
            ),
            (
              [payload.histograms.update_can_use_bits_notify],
              [
                'CanUseBits',
                'NoBits_NotWindows',
                'NoBits_FeatureOff',
                'NoBits_Pref',
                'NoBits_Proxy',
                'NoBits_OtherUser'
              ]
            ),
            (
              [payload.histograms.webext_browseraction_popup_preload_result_count],
              ['popupShown', 'clearAfterHover', 'clearAfterMousedown']
            )
          ]
        ),
        UNNEST(histograms) AS histogram
    ) AS categorical_histograms,
    ARRAY(
      SELECT AS STRUCT
        udf_null_if_empty_list(
          ARRAY(
            SELECT AS STRUCT
              key,
              ARRAY(
                SELECT AS STRUCT
                  IFNULL(labels[SAFE_OFFSET(_.key)], 'spill') AS key,
                  _.value
                FROM
                  UNNEST(mozfun.hist.extract(value).values) AS _
              ) AS value
            FROM
              UNNEST(histogram)
          )
        ) AS list
      FROM
        UNNEST(
          [
            STRUCT(
              [
                -- format:off
                STRUCT(payload.processes.content.keyed_histograms.uptake_remote_content_result_1 AS histogram),
                STRUCT(payload.processes.gpu.keyed_histograms.uptake_remote_content_result_1 AS histogram),
                STRUCT(payload.keyed_histograms.uptake_remote_content_result_1 AS histogram)
                -- format:on
              ] AS histograms,
              [
                'up_to_date',
                'success',
                'backoff',
                'pref_disabled',
                'parse_error',
                'content_error',
                'sign_error',
                'sign_retry_error',
                'conflict_error',
                'sync_error',
                'apply_error',
                'server_error',
                'certificate_error',
                'download_error',
                'timeout_error',
                'network_error',
                'offline_error',
                'cleanup_error',
                'unknown_error',
                'custom_1_error',
                'custom_2_error',
                'custom_3_error',
                'custom_4_error',
                'custom_5_error'
              ] AS labels
            )
          ]
        ),
        UNNEST(histograms)
    ) AS keyed_categorical_histograms
  FROM
    `moz-fx-data-shared-prod.telemetry.main`
  WHERE
    DATE(submission_timestamp) = @submission_date
    AND normalized_app_name = 'Firefox'
    AND document_id IS NOT NULL
)
SELECT
  * EXCEPT (
    values_histograms,
    keyed_values_histograms,
    count_histograms,
    keyed_count_histograms,
    categorical_histograms,
    keyed_categorical_histograms
  ),
  -- format:off
  values_histograms[OFFSET(0)] AS histogram_parent_a11y_consumers,
  values_histograms[OFFSET(1)] AS histogram_parent_cert_validation_success_by_ca,
  values_histograms[OFFSET(2)] AS histogram_content_cert_validation_success_by_ca,
  values_histograms[OFFSET(3)] AS histogram_parent_checkerboard_severity,
  values_histograms[OFFSET(4)] AS histogram_content_checkerboard_severity,
  values_histograms[OFFSET(5)] AS histogram_gpu_checkerboard_severity,
  values_histograms[OFFSET(6)] AS histogram_parent_composite_time,
  values_histograms[OFFSET(7)] AS histogram_content_composite_time,
  values_histograms[OFFSET(8)] AS histogram_gpu_composite_time,
  values_histograms[OFFSET(9)] AS histogram_gpu_content_frame_time,
  values_histograms[OFFSET(10)] AS histogram_parent_content_paint_time,
  values_histograms[OFFSET(11)] AS histogram_content_content_paint_time,
  values_histograms[OFFSET(12)] AS histogram_parent_cookie_behavior,
  values_histograms[OFFSET(13)] AS histogram_parent_cycle_collector_max_pause,
  values_histograms[OFFSET(14)] AS histogram_content_cycle_collector_max_pause,
  values_histograms[OFFSET(15)] AS histogram_parent_devtools_accessibility_picker_time_active_seconds,
  values_histograms[OFFSET(16)] AS histogram_content_devtools_accessibility_picker_time_active_seconds,
  values_histograms[OFFSET(17)] AS histogram_parent_devtools_accessibility_service_time_active_seconds,
  values_histograms[OFFSET(18)] AS histogram_content_devtools_accessibility_service_time_active_seconds,
  values_histograms[OFFSET(19)] AS histogram_parent_devtools_accessibility_time_active_seconds,
  values_histograms[OFFSET(10)] AS histogram_content_devtools_accessibility_time_active_seconds,
  values_histograms[OFFSET(21)] AS histogram_parent_devtools_fonteditor_n_fonts_rendered,
  values_histograms[OFFSET(22)] AS histogram_parent_devtools_fonteditor_n_font_axes,
  values_histograms[OFFSET(23)] AS histogram_parent_devtools_toolbox_time_active_seconds,
  values_histograms[OFFSET(24)] AS histogram_content_devtools_toolbox_time_active_seconds,
  values_histograms[OFFSET(25)] AS histogram_parent_dns_failed_lookup_time,
  values_histograms[OFFSET(26)] AS histogram_parent_dns_lookup_time,
  values_histograms[OFFSET(27)] AS histogram_parent_fx_new_window_ms,
  values_histograms[OFFSET(28)] AS histogram_parent_fx_page_load_ms_2,
  values_histograms[OFFSET(29)] AS histogram_content_fx_page_load_ms_2,
  values_histograms[OFFSET(30)] AS histogram_parent_fx_session_restore_restore_window_ms,
  values_histograms[OFFSET(31)] AS histogram_parent_fx_session_restore_startup_init_session_ms,
  values_histograms[OFFSET(32)] AS histogram_parent_fx_session_restore_startup_onload_initial_window_ms,
  values_histograms[OFFSET(33)] AS histogram_parent_fx_tab_close_time_anim_ms,
  values_histograms[OFFSET(34)] AS histogram_parent_fx_tab_switch_total_e10s_ms,
  values_histograms[OFFSET(35)] AS histogram_parent_fx_tab_switch_update_ms,
  values_histograms[OFFSET(36)] AS histogram_parent_fx_urlbar_selected_result_index,
  values_histograms[OFFSET(37)] AS histogram_content_fx_urlbar_selected_result_index,
  values_histograms[OFFSET(38)] AS histogram_parent_fx_urlbar_selected_result_type,
  values_histograms[OFFSET(39)] AS histogram_content_fx_urlbar_selected_result_type,
  values_histograms[OFFSET(40)] AS histogram_parent_gc_animation_ms,
  values_histograms[OFFSET(41)] AS histogram_content_gc_animation_ms,
  values_histograms[OFFSET(42)] AS histogram_parent_gc_max_pause_ms_2,
  values_histograms[OFFSET(43)] AS histogram_content_gc_max_pause_ms_2,
  values_histograms[OFFSET(44)] AS histogram_parent_geolocation_request_granted,
  values_histograms[OFFSET(45)] AS histogram_content_geolocation_request_granted,
  values_histograms[OFFSET(46)] AS histogram_parent_ghost_windows,
  values_histograms[OFFSET(47)] AS histogram_content_ghost_windows,
  values_histograms[OFFSET(48)] AS histogram_parent_gpu_process_initialization_time_ms,
  values_histograms[OFFSET(49)] AS histogram_content_gpu_process_initialization_time_ms,
  values_histograms[OFFSET(50)] AS histogram_gpu_gpu_process_initialization_time_ms,
  values_histograms[OFFSET(51)] AS histogram_parent_gpu_process_launch_time_ms_2,
  values_histograms[OFFSET(52)] AS histogram_content_gpu_process_launch_time_ms_2,
  values_histograms[OFFSET(53)] AS histogram_parent_http_channel_disposition,
  values_histograms[OFFSET(54)] AS histogram_content_http_channel_disposition,
  values_histograms[OFFSET(55)] AS histogram_parent_http_pageload_is_ssl,
  values_histograms[OFFSET(56)] AS histogram_content_http_pageload_is_ssl,
  values_histograms[OFFSET(57)] AS histogram_parent_http_transaction_is_ssl,
  values_histograms[OFFSET(58)] AS histogram_content_http_transaction_is_ssl,
  values_histograms[OFFSET(59)] AS histogram_parent_input_event_response_coalesced_ms,
  values_histograms[OFFSET(60)] AS histogram_content_input_event_response_coalesced_ms,
  values_histograms[OFFSET(61)] AS histogram_parent_memory_heap_allocated,
  values_histograms[OFFSET(62)] AS histogram_content_memory_heap_allocated,
  values_histograms[OFFSET(63)] AS histogram_parent_memory_resident_fast,
  values_histograms[OFFSET(64)] AS histogram_content_memory_resident_fast,
  values_histograms[OFFSET(65)] AS histogram_parent_memory_total,
  values_histograms[OFFSET(66)] AS histogram_parent_memory_unique,
  values_histograms[OFFSET(67)] AS histogram_content_memory_unique,
  values_histograms[OFFSET(68)] AS histogram_content_memory_unique_content_startup,
  values_histograms[OFFSET(69)] AS histogram_parent_memory_vsize,
  values_histograms[OFFSET(70)] AS histogram_content_memory_vsize,
  values_histograms[OFFSET(71)] AS histogram_parent_memory_vsize_max_contiguous,
  values_histograms[OFFSET(72)] AS histogram_content_memory_vsize_max_contiguous,
  values_histograms[OFFSET(73)] AS histogram_parent_network_cache_metadata_first_read_time_ms,
  values_histograms[OFFSET(74)] AS histogram_content_network_cache_metadata_first_read_time_ms,
  values_histograms[OFFSET(75)] AS histogram_parent_network_cache_v2_hit_time_ms,
  values_histograms[OFFSET(76)] AS histogram_content_network_cache_v2_hit_time_ms,
  values_histograms[OFFSET(77)] AS histogram_parent_network_cache_v2_miss_time_ms,
  values_histograms[OFFSET(78)] AS histogram_content_network_cache_v2_miss_time_ms,
  values_histograms[OFFSET(79)] AS histogram_parent_places_autocomplete_6_first_results_time_ms,
  values_histograms[OFFSET(80)] AS histogram_parent_plugin_shutdown_ms,
  values_histograms[OFFSET(81)] AS histogram_content_plugin_shutdown_ms,
  values_histograms[OFFSET(82)] AS histogram_parent_pwmgr_blocklist_num_sites,
  values_histograms[OFFSET(83)] AS histogram_parent_pwmgr_form_autofill_result,
  values_histograms[OFFSET(84)] AS histogram_content_pwmgr_form_autofill_result,
  values_histograms[OFFSET(85)] AS histogram_parent_pwmgr_login_last_used_days,
  values_histograms[OFFSET(86)] AS histogram_parent_pwmgr_login_page_safety,
  values_histograms[OFFSET(87)] AS histogram_content_pwmgr_login_page_safety,
  values_histograms[OFFSET(88)] AS histogram_parent_pwmgr_manage_opened,
  values_histograms[OFFSET(89)] AS histogram_parent_pwmgr_manage_visibility_toggled,
  values_histograms[OFFSET(90)] AS histogram_parent_pwmgr_num_passwords_per_hostname,
  values_histograms[OFFSET(91)] AS histogram_parent_pwmgr_num_saved_passwords,
  values_histograms[OFFSET(92)] AS histogram_parent_pwmgr_prompt_remember_action,
  values_histograms[OFFSET(93)] AS histogram_parent_pwmgr_prompt_update_action,
  values_histograms[OFFSET(94)] AS histogram_parent_pwmgr_saving_enabled,
  values_histograms[OFFSET(95)] AS histogram_parent_search_service_init_ms,
  values_histograms[OFFSET(96)] AS histogram_parent_ssl_handshake_result,
  values_histograms[OFFSET(97)] AS histogram_content_ssl_handshake_result,
  values_histograms[OFFSET(98)] AS histogram_parent_ssl_handshake_version,
  values_histograms[OFFSET(99)] AS histogram_content_ssl_handshake_version,
  values_histograms[OFFSET(100)] AS histogram_parent_ssl_tls12_intolerance_reason_pre,
  values_histograms[OFFSET(101)] AS histogram_content_ssl_tls12_intolerance_reason_pre,
  values_histograms[OFFSET(102)] AS histogram_parent_ssl_tls13_intolerance_reason_pre,
  values_histograms[OFFSET(103)] AS histogram_content_ssl_tls13_intolerance_reason_pre,
  values_histograms[OFFSET(104)] AS histogram_content_time_to_dom_complete_ms,
  values_histograms[OFFSET(105)] AS histogram_content_time_to_dom_content_loaded_end_ms,
  values_histograms[OFFSET(106)] AS histogram_content_time_to_dom_content_loaded_start_ms,
  values_histograms[OFFSET(107)] AS histogram_content_time_to_dom_interactive_ms,
  values_histograms[OFFSET(108)] AS histogram_content_time_to_dom_loading_ms,
  values_histograms[OFFSET(109)] AS histogram_parent_time_to_first_click_ms,
  values_histograms[OFFSET(110)] AS histogram_content_time_to_first_click_ms,
  values_histograms[OFFSET(111)] AS histogram_parent_time_to_first_interaction_ms,
  values_histograms[OFFSET(112)] AS histogram_content_time_to_first_interaction_ms,
  values_histograms[OFFSET(113)] AS histogram_parent_time_to_first_key_input_ms,
  values_histograms[OFFSET(114)] AS histogram_content_time_to_first_key_input_ms,
  values_histograms[OFFSET(115)] AS histogram_parent_time_to_first_mouse_move_ms,
  values_histograms[OFFSET(116)] AS histogram_content_time_to_first_mouse_move_ms,
  values_histograms[OFFSET(117)] AS histogram_parent_time_to_first_scroll_ms,
  values_histograms[OFFSET(118)] AS histogram_content_time_to_first_scroll_ms,
  values_histograms[OFFSET(119)] AS histogram_content_time_to_load_event_end_ms,
  values_histograms[OFFSET(120)] AS histogram_content_time_to_load_event_start_ms,
  values_histograms[OFFSET(121)] AS histogram_parent_time_to_non_blank_paint_ms,
  values_histograms[OFFSET(122)] AS histogram_content_time_to_non_blank_paint_ms,
  values_histograms[OFFSET(123)] AS histogram_parent_time_to_response_start_ms,
  values_histograms[OFFSET(124)] AS histogram_content_time_to_response_start_ms,
  values_histograms[OFFSET(125)] AS histogram_parent_touch_enabled_device,
  values_histograms[OFFSET(126)] AS histogram_parent_tracking_protection_enabled,
  values_histograms[OFFSET(127)] AS histogram_parent_update_download_code_partial,
  values_histograms[OFFSET(128)] AS histogram_parent_update_download_code_complete,
  values_histograms[OFFSET(129)] AS histogram_parent_update_bits_result_partial,
  values_histograms[OFFSET(130)] AS histogram_parent_update_bits_result_complete,
  values_histograms[OFFSET(131)] AS histogram_parent_update_state_code_partial_stage,
  values_histograms[OFFSET(132)] AS histogram_parent_update_state_code_complete_stage,
  values_histograms[OFFSET(133)] AS histogram_parent_update_status_error_code_complete_stage,
  values_histograms[OFFSET(134)] AS histogram_parent_update_status_error_code_partial_stage,
  values_histograms[OFFSET(135)] AS histogram_parent_update_state_code_partial_startup,
  values_histograms[OFFSET(136)] AS histogram_parent_update_state_code_complete_startup,
  values_histograms[OFFSET(137)] AS histogram_parent_update_status_error_code_partial_startup,
  values_histograms[OFFSET(138)] AS histogram_parent_update_status_error_code_complete_startup,
  values_histograms[OFFSET(139)] AS histogram_parent_webext_background_page_load_ms,
  values_histograms[OFFSET(140)] AS histogram_parent_webext_browseraction_popup_open_ms,
  values_histograms[OFFSET(141)] AS histogram_parent_webext_content_script_injection_ms,
  values_histograms[OFFSET(142)] AS histogram_content_webext_content_script_injection_ms,
  values_histograms[OFFSET(143)] AS histogram_parent_webext_extension_startup_ms,
  values_histograms[OFFSET(144)] AS histogram_parent_webext_pageaction_popup_open_ms,
  values_histograms[OFFSET(145)] AS histogram_parent_webext_storage_local_get_ms,
  values_histograms[OFFSET(146)] AS histogram_content_webext_storage_local_get_ms,
  values_histograms[OFFSET(147)] AS histogram_parent_webext_storage_local_set_ms,
  values_histograms[OFFSET(148)] AS histogram_content_webext_storage_local_set_ms,
  values_histograms[OFFSET(149)] AS histogram_parent_webvr_time_spent_viewing_in_2d,
  values_histograms[OFFSET(150)] AS histogram_content_webvr_time_spent_viewing_in_2d,
  values_histograms[OFFSET(151)] AS histogram_parent_webvr_time_spent_viewing_in_oculus,
  values_histograms[OFFSET(152)] AS histogram_gpu_webvr_time_spent_viewing_in_oculus,
  values_histograms[OFFSET(153)] AS histogram_parent_webvr_time_spent_viewing_in_openvr,
  values_histograms[OFFSET(154)] AS histogram_gpu_webvr_time_spent_viewing_in_openvr,
  values_histograms[OFFSET(155)] AS histogram_parent_webvr_users_view_in,
  values_histograms[OFFSET(156)] AS histogram_gpu_webvr_users_view_in,
  values_histograms[OFFSET(157)] AS histogram_content_webvr_users_view_in,
  values_histograms[OFFSET(158)] AS histogram_parent_a11y_instantiated_flag,
  values_histograms[OFFSET(159)] AS histogram_parent_search_reset_result,
  values_histograms[OFFSET(160)] AS histogram_content_search_reset_result,
  values_histograms[OFFSET(161)] AS histogram_parent_gc_max_pause_ms,
  values_histograms[OFFSET(162)] AS histogram_content_gc_max_pause_ms,
  values_histograms[OFFSET(163)] AS histogram_parent_content_frame_time,
  values_histograms[OFFSET(164)] AS histogram_content_tracking_protection_enabled,
  values_histograms[OFFSET(165)] AS histogram_content_places_autocomplete_6_first_results_time_ms,
  values_histograms[OFFSET(166)] AS histogram_content_memory_total,
  values_histograms[OFFSET(167)] AS histogram_content_fx_session_restore_restore_window_ms,
  values_histograms[OFFSET(168)] AS histogram_content_fx_session_restore_startup_init_session_ms,
  values_histograms[OFFSET(169)] AS histogram_content_fx_session_restore_startup_onload_initial_window_ms,
  values_histograms[OFFSET(170)] AS histogram_content_fx_tab_close_time_anim_ms,
  values_histograms[OFFSET(171)] AS histogram_content_fx_tab_switch_update_ms,
  values_histograms[OFFSET(172)] AS histogram_content_fx_new_window_ms,
  values_histograms[OFFSET(173)] AS histogram_content_search_service_init_ms,
  keyed_values_histograms[OFFSET(0)] AS histogram_content_fx_urlbar_selected_result_index_by_type,
  keyed_values_histograms[OFFSET(1)] AS histogram_parent_fx_urlbar_selected_result_index_by_type,
  keyed_values_histograms[OFFSET(2)] AS histogram_content_ipc_read_main_thread_latency_ms,
  keyed_values_histograms[OFFSET(3)] AS histogram_gpu_ipc_read_main_thread_latency_ms,
  keyed_values_histograms[OFFSET(4)] AS histogram_parent_ipc_read_main_thread_latency_ms,
  keyed_values_histograms[OFFSET(5)] AS histogram_parent_memory_distribution_among_content,
  keyed_values_histograms[OFFSET(6)] AS histogram_content_memory_distribution_among_content,
  count_histograms[OFFSET( 0)].histogram AS histogram_content_devtools_aboutdebugging_opened_count,
  count_histograms[OFFSET( 1)].histogram AS histogram_content_devtools_animationinspector_opened_count,
  count_histograms[OFFSET( 2)].histogram AS histogram_content_devtools_browserconsole_opened_count,
  count_histograms[OFFSET( 3)].histogram AS histogram_content_devtools_canvasdebugger_opened_count,
  count_histograms[OFFSET( 4)].histogram AS histogram_content_devtools_computedview_opened_count,
  count_histograms[OFFSET( 5)].histogram AS histogram_content_devtools_custom_opened_count,
  count_histograms[OFFSET( 6)].histogram AS histogram_content_devtools_dom_opened_count,
  count_histograms[OFFSET( 7)].histogram AS histogram_content_devtools_eyedropper_opened_count,
  count_histograms[OFFSET( 8)].histogram AS histogram_content_devtools_fontinspector_opened_count,
  count_histograms[OFFSET( 9)].histogram AS histogram_content_devtools_inspector_opened_count,
  count_histograms[OFFSET(10)].histogram AS histogram_content_devtools_jsbrowserdebugger_opened_count,
  count_histograms[OFFSET(11)].histogram AS histogram_content_devtools_jsdebugger_opened_count,
  count_histograms[OFFSET(12)].histogram AS histogram_content_devtools_jsprofiler_opened_count,
  count_histograms[OFFSET(13)].histogram AS histogram_content_devtools_layoutview_opened_count,
  count_histograms[OFFSET(14)].histogram AS histogram_content_devtools_memory_opened_count,
  count_histograms[OFFSET(15)].histogram AS histogram_content_devtools_menu_eyedropper_opened_count,
  count_histograms[OFFSET(16)].histogram AS histogram_content_devtools_netmonitor_opened_count,
  count_histograms[OFFSET(17)].histogram AS histogram_content_devtools_options_opened_count,
  count_histograms[OFFSET(18)].histogram AS histogram_content_devtools_paintflashing_opened_count,
  count_histograms[OFFSET(19)].histogram AS histogram_content_devtools_picker_eyedropper_opened_count,
  count_histograms[OFFSET(20)].histogram AS histogram_content_devtools_responsive_opened_count,
  count_histograms[OFFSET(21)].histogram AS histogram_content_devtools_ruleview_opened_count,
  count_histograms[OFFSET(22)].histogram AS histogram_content_devtools_scratchpad_opened_count,
  count_histograms[OFFSET(23)].histogram AS histogram_content_devtools_scratchpad_window_opened_count,
  count_histograms[OFFSET(24)].histogram AS histogram_content_devtools_shadereditor_opened_count,
  count_histograms[OFFSET(25)].histogram AS histogram_content_devtools_storage_opened_count,
  count_histograms[OFFSET(26)].histogram AS histogram_content_devtools_styleeditor_opened_count,
  count_histograms[OFFSET(27)].histogram AS histogram_content_devtools_webaudioeditor_opened_count,
  count_histograms[OFFSET(28)].histogram AS histogram_content_devtools_webconsole_opened_count,
  count_histograms[OFFSET(29)].histogram AS histogram_content_devtools_webide_opened_count,
  count_histograms[OFFSET(30)].histogram AS histogram_content_number_of_profiles,
  count_histograms[OFFSET(31)].histogram AS histogram_content_pwmgr_manage_copied_password,
  count_histograms[OFFSET(32)].histogram AS histogram_content_pwmgr_manage_copied_username,
  count_histograms[OFFSET(33)].histogram AS histogram_content_pwmgr_manage_deleted,
  count_histograms[OFFSET(34)].histogram AS histogram_parent_devtools_aboutdebugging_opened_count,
  count_histograms[OFFSET(35)].histogram AS histogram_parent_devtools_animationinspector_opened_count,
  count_histograms[OFFSET(36)].histogram AS histogram_parent_devtools_browserconsole_opened_count,
  count_histograms[OFFSET(37)].histogram AS histogram_parent_devtools_canvasdebugger_opened_count,
  count_histograms[OFFSET(38)].histogram AS histogram_parent_devtools_computedview_opened_count,
  count_histograms[OFFSET(39)].histogram AS histogram_parent_devtools_custom_opened_count,
  count_histograms[OFFSET(40)].histogram AS histogram_parent_devtools_dom_opened_count,
  count_histograms[OFFSET(41)].histogram AS histogram_parent_devtools_eyedropper_opened_count,
  count_histograms[OFFSET(42)].histogram AS histogram_parent_devtools_fontinspector_opened_count,
  count_histograms[OFFSET(43)].histogram AS histogram_parent_devtools_inspector_opened_count,
  count_histograms[OFFSET(44)].histogram AS histogram_parent_devtools_jsbrowserdebugger_opened_count,
  count_histograms[OFFSET(45)].histogram AS histogram_parent_devtools_jsdebugger_opened_count,
  count_histograms[OFFSET(46)].histogram AS histogram_parent_devtools_jsprofiler_opened_count,
  count_histograms[OFFSET(47)].histogram AS histogram_parent_devtools_layoutview_opened_count,
  count_histograms[OFFSET(48)].histogram AS histogram_parent_devtools_memory_opened_count,
  count_histograms[OFFSET(49)].histogram AS histogram_parent_devtools_menu_eyedropper_opened_count,
  count_histograms[OFFSET(50)].histogram AS histogram_parent_devtools_netmonitor_opened_count,
  count_histograms[OFFSET(51)].histogram AS histogram_parent_devtools_options_opened_count,
  count_histograms[OFFSET(52)].histogram AS histogram_parent_devtools_paintflashing_opened_count,
  count_histograms[OFFSET(53)].histogram AS histogram_parent_devtools_picker_eyedropper_opened_count,
  count_histograms[OFFSET(54)].histogram AS histogram_parent_devtools_responsive_opened_count,
  count_histograms[OFFSET(55)].histogram AS histogram_parent_devtools_ruleview_opened_count,
  count_histograms[OFFSET(56)].histogram AS histogram_parent_devtools_scratchpad_opened_count,
  count_histograms[OFFSET(57)].histogram AS histogram_parent_devtools_scratchpad_window_opened_count,
  count_histograms[OFFSET(58)].histogram AS histogram_parent_devtools_shadereditor_opened_count,
  count_histograms[OFFSET(59)].histogram AS histogram_parent_devtools_storage_opened_count,
  count_histograms[OFFSET(60)].histogram AS histogram_parent_devtools_styleeditor_opened_count,
  count_histograms[OFFSET(61)].histogram AS histogram_parent_devtools_webaudioeditor_opened_count,
  count_histograms[OFFSET(62)].histogram AS histogram_parent_devtools_webconsole_opened_count,
  count_histograms[OFFSET(63)].histogram AS histogram_parent_devtools_webide_opened_count,
  count_histograms[OFFSET(64)].histogram AS histogram_parent_number_of_profiles,
  count_histograms[OFFSET(65)].histogram AS histogram_parent_pwmgr_manage_copied_password,
  count_histograms[OFFSET(66)].histogram AS histogram_parent_pwmgr_manage_copied_username,
  count_histograms[OFFSET(67)].histogram AS histogram_parent_pwmgr_manage_deleted,
  count_histograms[OFFSET(68)].histogram AS histogram_parent_devtools_developertoolbar_opened_count,
  count_histograms[OFFSET(69)].histogram AS histogram_content_devtools_developertoolbar_opened_count,
  keyed_count_histograms[OFFSET(0)] AS histogram_content_sandbox_rejected_syscalls,
  keyed_count_histograms[OFFSET(1)] AS histogram_parent_sandbox_rejected_syscalls,
  categorical_histograms[OFFSET(0)] AS histogram_content_fx_searchbar_selected_result_method,
  categorical_histograms[OFFSET(1)] AS histogram_parent_fx_searchbar_selected_result_method,
  categorical_histograms[OFFSET(2)] AS histogram_parent_devtools_about_devtools_opened_key,
  categorical_histograms[OFFSET(3)] AS histogram_parent_devtools_about_devtools_opened_reason,
  categorical_histograms[OFFSET(4)] AS histogram_parent_devtools_entry_point,
  categorical_histograms[OFFSET(5)] AS histogram_parent_devtools_fonteditor_font_type_displayed,
  categorical_histograms[OFFSET(6)] AS histogram_content_fx_urlbar_selected_result_method,
  categorical_histograms[OFFSET(7)] AS histogram_parent_fx_urlbar_selected_result_method,
  categorical_histograms[OFFSET(8)] AS histogram_parent_update_can_use_bits_notify,
  categorical_histograms[OFFSET(9)] AS histogram_parent_webext_browseraction_popup_preload_result_count,
  keyed_categorical_histograms[OFFSET(0)] AS histogram_content_uptake_remote_content_result_1,
  keyed_categorical_histograms[OFFSET(1)] AS histogram_gpu_uptake_remote_content_result_1,
  keyed_categorical_histograms[OFFSET(2)] AS histogram_parent_uptake_remote_content_result_1
  -- format:on
FROM
  histogram_lists
