-- Generated via bigquery_etl.glean_usage
CREATE OR REPLACE VIEW
  `moz-fx-data-shared-prod.org_mozilla_fenix_nightly.baseline_clients_daily`
AS
SELECT
  *
FROM
  `moz-fx-data-shared-prod.org_mozilla_fenix_nightly_derived.baseline_clients_daily_v1`
