CREATE OR REPLACE VIEW
  `moz-fx-data-shared-prod.messaging_system.cfr_exact_mau28_by_dimensions`
AS
SELECT
  *
FROM
  `moz-fx-data-shared-prod.messaging_system_derived.cfr_exact_mau28_by_dimensions_v1`
