-- Query generated by: templates/unnest_parquet_view.sql.py experiments_aggregates_v1 telemetry_derived.experiments_aggregates_v1
CREATE OR REPLACE VIEW
  `moz-fx-data-shared-prod.telemetry.experiments_aggregates_v1`
AS
SELECT
  * REPLACE (
    histogram.key_value AS histogram,
    ARRAY(SELECT * FROM UNNEST(statistics.list)) AS statistics
  )
FROM
  `moz-fx-data-shared-prod.telemetry_derived.experiments_aggregates_v1`
