CREATE TABLE IF NOT EXISTS
  `moz-fx-data-marketing-prod.ga_derived.www_site_hits_v1`(
    date DATE,
    visit_identifier STRING,
    full_visitor_id STRING,
    visit_start_time INT64,
    page_path STRING,
    page_path_level1 STRING,
    page_level_1 STRING,
    page_level_2 STRING,
    page_level_3 STRING,
    page_level_4 STRING,
    page_level_5 STRING,
    hit_type STRING,
    is_exit BOOLEAN,
    is_entrance BOOLEAN,
    hit_number INT64,
    event_category STRING,
    event_label STRING,
    event_action STRING,
    device_category STRING,
    operating_system STRING,
    LANGUAGE STRING,
    browser STRING,
    browser_version STRING,
    country STRING,
    source STRING,
    medium STRING,
    campaign STRING,
    ad_content STRING,
    visits INT64,
    bounces INT64,
    hit_time FLOAT64,
    first_interaction INT64,
    last_interaction FLOAT64,
    entrances INT64,
    exits INT64,
    event_id STRING
  )
PARTITION BY
  date
