CREATE OR REPLACE VIEW
  `moz-fx-data-shared-prod.messaging_system.onboarding_exact_mau28_by_dimensions`
AS
SELECT
  *
FROM
  `moz-fx-data-shared-prod.messaging_system_derived.onboarding_exact_mau28_by_dimensions_v1`
