-- Query generated by: templates/unnest_parquet_view.sql.py main_summary_v4 telemetry_derived.main_summary_v4
CREATE OR REPLACE VIEW
  `moz-fx-data-shared-prod.telemetry.main_summary_v4`
AS
SELECT
  submission_date AS submission_date_s3,
  * EXCEPT (
    histogram_content_devtools_computedview_opened_count,
    histogram_content_devtools_fontinspector_opened_count,
    histogram_content_devtools_storage_opened_count,
    scalar_parent_datasanitization_privacy_clearOnShutdown_downloads,
    histogram_content_devtools_eyedropper_opened_count,
    histogram_content_devtools_netmonitor_opened_count,
    scalar_parent_pdf_viewer_fallback_shown,
    histogram_content_devtools_paintflashing_opened_count,
    histogram_content_devtools_custom_opened_count,
    histogram_content_devtools_aboutdebugging_opened_count,
    histogram_content_devtools_layoutview_opened_count,
    scalar_parent_telemetry_ecosystem_new_send_time,
    scalar_parent_media_blocked_autoplay_no_audio_track_count,
    push_api_notification_received,
    histogram_content_devtools_jsbrowserdebugger_opened_count,
    scalar_parent_webrtc_peerconnection_datachannel_max_life_used,
    histogram_content_pwmgr_manage_deleted,
    scalar_parent_webrtc_peerconnection_promise_stats_used,
    scalar_content_webrtc_peerconnection_promise_and_callback_stats_used,
    scalar_parent_devtools_aboutdevtools_installed,
    histogram_content_devtools_webide_opened_count,
    histogram_content_devtools_picker_eyedropper_opened_count,
    scalar_parent_devtools_aboutdevtools_opened,
    scalar_parent_datasanitization_privacy_clearOnShutdown_offlineApps,
    scalar_parent_datasanitization_privacy_clearOnShutdown_siteSettings,
    scalar_parent_mediarecorder_recording_count,
    scalar_parent_sw_cors_res_for_so_req_count,
    scalar_parent_telemetry_ecosystem_old_send_time,
    scalar_parent_webrtc_peerconnection_legacy_callback_stats_used,
    scalar_parent_idb_type_temporary_count,
    scalar_gpu_telemetry_profile_directory_scans,
    scalar_parent_telemetry_persistence_timer_hit_count,
    histogram_content_devtools_scratchpad_window_opened_count,
    scalar_parent_datasanitization_privacy_clearOnShutdown_cookies,
    histogram_content_devtools_memory_opened_count,
    scalar_parent_datasanitization_privacy_sanitize_sanitizeOnShutdown,
    histogram_content_devtools_webaudioeditor_opened_count,
    scalar_content_webrtc_peerconnection_datachannel_max_retx_and_life_used,
    scalar_parent_webrtc_hostnameobfuscation_disabled_failed,
    histogram_content_devtools_jsdebugger_opened_count,
    scalar_parent_webrtc_peerconnection_datachannel_max_retx_used,
    scalar_gpu_browser_usage_graphite,
    histogram_content_pwmgr_manage_copied_password,
    scalar_parent_browser_errors_sample_rate,
    scalar_parent_webrtc_hostnameobfuscation_enabled_succeeded,
    scalar_parent_datasanitization_privacy_clearOnShutdown_openWindows,
    scalar_parent_devtools_aboutdevtools_noinstall_exits,
    scalar_parent_datasanitization_privacy_clearOnShutdown_sessions,
    scalar_parent_webrtc_peerconnection_connected,
    scalar_content_webrtc_peerconnection_datachannel_max_retx_used,
    scalar_parent_webrtc_hostnameobfuscation_disabled_succeeded,
    scalar_parent_datasanitization_session_permission_exceptions,
    scalar_content_telemetry_profile_directory_scans,
    histogram_content_devtools_developertoolbar_opened_count,
    histogram_content_pwmgr_manage_copied_username,
    scalar_parent_telemetry_process_creation_timestamp_inconsistent,
    scalar_content_idb_type_temporary_count,
    scalar_parent_webrtc_peerconnection_datachannel_max_retx_and_life_used,
    scalar_parent_telemetry_pending_operations_highwatermark_reached,
    histogram_content_devtools_inspector_opened_count,
    histogram_content_number_of_profiles,
    scalar_content_media_autoplay_default_blocked,
    scalar_parent_devtools_application_opened_count,
    histogram_content_devtools_shadereditor_opened_count,
    scalar_parent_devtools_webreplay_save_recording,
    histogram_content_devtools_animationinspector_opened_count,
    histogram_content_devtools_dom_opened_count,
    histogram_content_devtools_styleeditor_opened_count,
    scalar_parent_webrtc_hostnameobfuscation_enabled_failed,
    histogram_content_devtools_ruleview_opened_count,
    scalar_parent_webrtc_peerconnection_promise_and_callback_stats_used,
    histogram_content_devtools_options_opened_count,
    scalar_parent_datasanitization_privacy_clearOnShutdown_cache,
    scalar_parent_datasanitization_privacy_clearOnShutdown_formdata,
    scalar_content_webrtc_peerconnection_legacy_callback_stats_used,
    histogram_content_devtools_responsive_opened_count,
    loop_activity_counter,
    scalar_parent_pdf_viewer_print,
    scalar_parent_browser_session_restore_tabbar_restore_clicked,
    histogram_content_devtools_webconsole_opened_count,
    histogram_content_devtools_scratchpad_opened_count,
    scalar_parent_datasanitization_privacy_clearOnShutdown_history,
    user_pref_browser_launcherprocess_enabled,
    histogram_content_devtools_canvasdebugger_opened_count,
    scalar_content_telemetry_process_creation_timestamp_inconsistent,
    scalar_content_sw_cors_res_for_so_req_count,
    scalar_parent_datasanitization_network_cookie_lifetimePolicy,
    scalar_content_media_blocked_autoplay_no_audio_track_count,
    histogram_content_devtools_jsprofiler_opened_count
  ) REPLACE(
    histogram_content_cert_validation_success_by_ca.list AS histogram_content_cert_validation_success_by_ca,
    histogram_content_checkerboard_severity.list AS histogram_content_checkerboard_severity,
    histogram_content_composite_time.list AS histogram_content_composite_time,
    histogram_content_content_paint_time.list AS histogram_content_content_paint_time,
    histogram_content_cycle_collector_max_pause.list AS histogram_content_cycle_collector_max_pause,
    histogram_content_devtools_accessibility_picker_time_active_seconds.list AS histogram_content_devtools_accessibility_picker_time_active_seconds,
    histogram_content_devtools_accessibility_service_time_active_seconds.list AS histogram_content_devtools_accessibility_service_time_active_seconds,
    histogram_content_devtools_accessibility_time_active_seconds.list AS histogram_content_devtools_accessibility_time_active_seconds,
    histogram_content_devtools_toolbox_time_active_seconds.list AS histogram_content_devtools_toolbox_time_active_seconds,
    histogram_content_fx_new_window_ms.list AS histogram_content_fx_new_window_ms,
    histogram_content_fx_page_load_ms_2.list AS histogram_content_fx_page_load_ms_2,
    histogram_content_fx_searchbar_selected_result_method.list AS histogram_content_fx_searchbar_selected_result_method,
    histogram_content_fx_session_restore_restore_window_ms.list AS histogram_content_fx_session_restore_restore_window_ms,
    histogram_content_fx_session_restore_startup_init_session_ms.list AS histogram_content_fx_session_restore_startup_init_session_ms,
    histogram_content_fx_session_restore_startup_onload_initial_window_ms.list AS histogram_content_fx_session_restore_startup_onload_initial_window_ms,
    histogram_content_fx_tab_close_time_anim_ms.list AS histogram_content_fx_tab_close_time_anim_ms,
    histogram_content_fx_tab_switch_update_ms.list AS histogram_content_fx_tab_switch_update_ms,
    histogram_content_fx_urlbar_selected_result_index.list AS histogram_content_fx_urlbar_selected_result_index,
    histogram_content_fx_urlbar_selected_result_index_by_type.list AS histogram_content_fx_urlbar_selected_result_index_by_type,
    histogram_content_fx_urlbar_selected_result_method.list AS histogram_content_fx_urlbar_selected_result_method,
    histogram_content_fx_urlbar_selected_result_type.list AS histogram_content_fx_urlbar_selected_result_type,
    histogram_content_gc_animation_ms.list AS histogram_content_gc_animation_ms,
    histogram_content_gc_max_pause_ms.list AS histogram_content_gc_max_pause_ms,
    histogram_content_gc_max_pause_ms_2.list AS histogram_content_gc_max_pause_ms_2,
    histogram_content_geolocation_request_granted.list AS histogram_content_geolocation_request_granted,
    histogram_content_ghost_windows.list AS histogram_content_ghost_windows,
    histogram_content_gpu_process_initialization_time_ms.list AS histogram_content_gpu_process_initialization_time_ms,
    histogram_content_gpu_process_launch_time_ms_2.list AS histogram_content_gpu_process_launch_time_ms_2,
    histogram_content_http_channel_disposition.list AS histogram_content_http_channel_disposition,
    histogram_content_http_pageload_is_ssl.list AS histogram_content_http_pageload_is_ssl,
    histogram_content_http_transaction_is_ssl.list AS histogram_content_http_transaction_is_ssl,
    histogram_content_input_event_response_coalesced_ms.list AS histogram_content_input_event_response_coalesced_ms,
    histogram_content_ipc_read_main_thread_latency_ms.list AS histogram_content_ipc_read_main_thread_latency_ms,
    histogram_content_memory_distribution_among_content.list AS histogram_content_memory_distribution_among_content,
    histogram_content_memory_heap_allocated.list AS histogram_content_memory_heap_allocated,
    histogram_content_memory_resident_fast.list AS histogram_content_memory_resident_fast,
    histogram_content_memory_total.list AS histogram_content_memory_total,
    histogram_content_memory_unique.list AS histogram_content_memory_unique,
    histogram_content_memory_unique_content_startup.list AS histogram_content_memory_unique_content_startup,
    histogram_content_memory_vsize.list AS histogram_content_memory_vsize,
    histogram_content_memory_vsize_max_contiguous.list AS histogram_content_memory_vsize_max_contiguous,
    histogram_content_network_cache_metadata_first_read_time_ms.list AS histogram_content_network_cache_metadata_first_read_time_ms,
    histogram_content_network_cache_v2_hit_time_ms.list AS histogram_content_network_cache_v2_hit_time_ms,
    histogram_content_network_cache_v2_miss_time_ms.list AS histogram_content_network_cache_v2_miss_time_ms,
    histogram_content_places_autocomplete_6_first_results_time_ms.list AS histogram_content_places_autocomplete_6_first_results_time_ms,
    histogram_content_plugin_shutdown_ms.list AS histogram_content_plugin_shutdown_ms,
    histogram_content_pwmgr_form_autofill_result.list AS histogram_content_pwmgr_form_autofill_result,
    histogram_content_pwmgr_login_page_safety.list AS histogram_content_pwmgr_login_page_safety,
    histogram_content_sandbox_rejected_syscalls.list AS histogram_content_sandbox_rejected_syscalls,
    histogram_content_search_reset_result.list AS histogram_content_search_reset_result,
    histogram_content_search_service_init_ms.list AS histogram_content_search_service_init_ms,
    histogram_content_ssl_handshake_result.list AS histogram_content_ssl_handshake_result,
    histogram_content_ssl_handshake_version.list AS histogram_content_ssl_handshake_version,
    histogram_content_ssl_tls12_intolerance_reason_pre.list AS histogram_content_ssl_tls12_intolerance_reason_pre,
    histogram_content_ssl_tls13_intolerance_reason_pre.list AS histogram_content_ssl_tls13_intolerance_reason_pre,
    histogram_content_time_to_dom_complete_ms.list AS histogram_content_time_to_dom_complete_ms,
    histogram_content_time_to_dom_content_loaded_end_ms.list AS histogram_content_time_to_dom_content_loaded_end_ms,
    histogram_content_time_to_dom_content_loaded_start_ms.list AS histogram_content_time_to_dom_content_loaded_start_ms,
    histogram_content_time_to_dom_interactive_ms.list AS histogram_content_time_to_dom_interactive_ms,
    histogram_content_time_to_dom_loading_ms.list AS histogram_content_time_to_dom_loading_ms,
    histogram_content_time_to_first_click_ms.list AS histogram_content_time_to_first_click_ms,
    histogram_content_time_to_first_interaction_ms.list AS histogram_content_time_to_first_interaction_ms,
    histogram_content_time_to_first_key_input_ms.list AS histogram_content_time_to_first_key_input_ms,
    histogram_content_time_to_first_mouse_move_ms.list AS histogram_content_time_to_first_mouse_move_ms,
    histogram_content_time_to_first_scroll_ms.list AS histogram_content_time_to_first_scroll_ms,
    histogram_content_time_to_load_event_end_ms.list AS histogram_content_time_to_load_event_end_ms,
    histogram_content_time_to_load_event_start_ms.list AS histogram_content_time_to_load_event_start_ms,
    histogram_content_time_to_non_blank_paint_ms.list AS histogram_content_time_to_non_blank_paint_ms,
    histogram_content_time_to_response_start_ms.list AS histogram_content_time_to_response_start_ms,
    histogram_content_tracking_protection_enabled.list AS histogram_content_tracking_protection_enabled,
    histogram_content_uptake_remote_content_result_1.list AS histogram_content_uptake_remote_content_result_1,
    histogram_content_webext_content_script_injection_ms.list AS histogram_content_webext_content_script_injection_ms,
    histogram_content_webext_storage_local_get_ms.list AS histogram_content_webext_storage_local_get_ms,
    histogram_content_webext_storage_local_set_ms.list AS histogram_content_webext_storage_local_set_ms,
    histogram_content_webvr_time_spent_viewing_in_2d.list AS histogram_content_webvr_time_spent_viewing_in_2d,
    histogram_content_webvr_users_view_in.list AS histogram_content_webvr_users_view_in,
    histogram_gpu_checkerboard_severity.list AS histogram_gpu_checkerboard_severity,
    histogram_gpu_composite_time.list AS histogram_gpu_composite_time,
    histogram_gpu_content_frame_time.list AS histogram_gpu_content_frame_time,
    histogram_gpu_gpu_process_initialization_time_ms.list AS histogram_gpu_gpu_process_initialization_time_ms,
    histogram_gpu_ipc_read_main_thread_latency_ms.list AS histogram_gpu_ipc_read_main_thread_latency_ms,
    histogram_gpu_uptake_remote_content_result_1.list AS histogram_gpu_uptake_remote_content_result_1,
    histogram_gpu_webvr_time_spent_viewing_in_oculus.list AS histogram_gpu_webvr_time_spent_viewing_in_oculus,
    histogram_gpu_webvr_time_spent_viewing_in_openvr.list AS histogram_gpu_webvr_time_spent_viewing_in_openvr,
    histogram_gpu_webvr_users_view_in.list AS histogram_gpu_webvr_users_view_in,
    histogram_parent_a11y_consumers.list AS histogram_parent_a11y_consumers,
    histogram_parent_a11y_instantiated_flag.list AS histogram_parent_a11y_instantiated_flag,
    histogram_parent_cert_validation_success_by_ca.list AS histogram_parent_cert_validation_success_by_ca,
    histogram_parent_checkerboard_severity.list AS histogram_parent_checkerboard_severity,
    histogram_parent_composite_time.list AS histogram_parent_composite_time,
    histogram_parent_content_frame_time.list AS histogram_parent_content_frame_time,
    histogram_parent_content_paint_time.list AS histogram_parent_content_paint_time,
    histogram_parent_cookie_behavior.list AS histogram_parent_cookie_behavior,
    histogram_parent_cycle_collector_max_pause.list AS histogram_parent_cycle_collector_max_pause,
    histogram_parent_devtools_about_devtools_opened_key.list AS histogram_parent_devtools_about_devtools_opened_key,
    histogram_parent_devtools_about_devtools_opened_reason.list AS histogram_parent_devtools_about_devtools_opened_reason,
    histogram_parent_devtools_accessibility_picker_time_active_seconds.list AS histogram_parent_devtools_accessibility_picker_time_active_seconds,
    histogram_parent_devtools_accessibility_service_time_active_seconds.list AS histogram_parent_devtools_accessibility_service_time_active_seconds,
    histogram_parent_devtools_accessibility_time_active_seconds.list AS histogram_parent_devtools_accessibility_time_active_seconds,
    histogram_parent_devtools_entry_point.list AS histogram_parent_devtools_entry_point,
    histogram_parent_devtools_fonteditor_font_type_displayed.list AS histogram_parent_devtools_fonteditor_font_type_displayed,
    histogram_parent_devtools_fonteditor_n_font_axes.list AS histogram_parent_devtools_fonteditor_n_font_axes,
    histogram_parent_devtools_fonteditor_n_fonts_rendered.list AS histogram_parent_devtools_fonteditor_n_fonts_rendered,
    histogram_parent_devtools_toolbox_time_active_seconds.list AS histogram_parent_devtools_toolbox_time_active_seconds,
    histogram_parent_dns_failed_lookup_time.list AS histogram_parent_dns_failed_lookup_time,
    histogram_parent_dns_lookup_time.list AS histogram_parent_dns_lookup_time,
    histogram_parent_fx_new_window_ms.list AS histogram_parent_fx_new_window_ms,
    histogram_parent_fx_page_load_ms_2.list AS histogram_parent_fx_page_load_ms_2,
    histogram_parent_fx_searchbar_selected_result_method.list AS histogram_parent_fx_searchbar_selected_result_method,
    histogram_parent_fx_session_restore_restore_window_ms.list AS histogram_parent_fx_session_restore_restore_window_ms,
    histogram_parent_fx_session_restore_startup_init_session_ms.list AS histogram_parent_fx_session_restore_startup_init_session_ms,
    histogram_parent_fx_session_restore_startup_onload_initial_window_ms.list AS histogram_parent_fx_session_restore_startup_onload_initial_window_ms,
    histogram_parent_fx_tab_close_time_anim_ms.list AS histogram_parent_fx_tab_close_time_anim_ms,
    histogram_parent_fx_tab_switch_total_e10s_ms.list AS histogram_parent_fx_tab_switch_total_e10s_ms,
    histogram_parent_fx_tab_switch_update_ms.list AS histogram_parent_fx_tab_switch_update_ms,
    histogram_parent_fx_urlbar_selected_result_index.list AS histogram_parent_fx_urlbar_selected_result_index,
    histogram_parent_fx_urlbar_selected_result_index_by_type.list AS histogram_parent_fx_urlbar_selected_result_index_by_type,
    histogram_parent_fx_urlbar_selected_result_method.list AS histogram_parent_fx_urlbar_selected_result_method,
    histogram_parent_fx_urlbar_selected_result_type.list AS histogram_parent_fx_urlbar_selected_result_type,
    histogram_parent_gc_animation_ms.list AS histogram_parent_gc_animation_ms,
    histogram_parent_gc_max_pause_ms.list AS histogram_parent_gc_max_pause_ms,
    histogram_parent_gc_max_pause_ms_2.list AS histogram_parent_gc_max_pause_ms_2,
    histogram_parent_geolocation_request_granted.list AS histogram_parent_geolocation_request_granted,
    histogram_parent_ghost_windows.list AS histogram_parent_ghost_windows,
    histogram_parent_gpu_process_initialization_time_ms.list AS histogram_parent_gpu_process_initialization_time_ms,
    histogram_parent_gpu_process_launch_time_ms_2.list AS histogram_parent_gpu_process_launch_time_ms_2,
    histogram_parent_http_channel_disposition.list AS histogram_parent_http_channel_disposition,
    histogram_parent_http_pageload_is_ssl.list AS histogram_parent_http_pageload_is_ssl,
    histogram_parent_http_transaction_is_ssl.list AS histogram_parent_http_transaction_is_ssl,
    histogram_parent_input_event_response_coalesced_ms.list AS histogram_parent_input_event_response_coalesced_ms,
    histogram_parent_ipc_read_main_thread_latency_ms.list AS histogram_parent_ipc_read_main_thread_latency_ms,
    histogram_parent_memory_distribution_among_content.list AS histogram_parent_memory_distribution_among_content,
    histogram_parent_memory_heap_allocated.list AS histogram_parent_memory_heap_allocated,
    histogram_parent_memory_resident_fast.list AS histogram_parent_memory_resident_fast,
    histogram_parent_memory_total.list AS histogram_parent_memory_total,
    histogram_parent_memory_unique.list AS histogram_parent_memory_unique,
    histogram_parent_memory_vsize.list AS histogram_parent_memory_vsize,
    histogram_parent_memory_vsize_max_contiguous.list AS histogram_parent_memory_vsize_max_contiguous,
    histogram_parent_network_cache_metadata_first_read_time_ms.list AS histogram_parent_network_cache_metadata_first_read_time_ms,
    histogram_parent_network_cache_v2_hit_time_ms.list AS histogram_parent_network_cache_v2_hit_time_ms,
    histogram_parent_network_cache_v2_miss_time_ms.list AS histogram_parent_network_cache_v2_miss_time_ms,
    histogram_parent_places_autocomplete_6_first_results_time_ms.list AS histogram_parent_places_autocomplete_6_first_results_time_ms,
    histogram_parent_plugin_shutdown_ms.list AS histogram_parent_plugin_shutdown_ms,
    histogram_parent_pwmgr_blocklist_num_sites.list AS histogram_parent_pwmgr_blocklist_num_sites,
    histogram_parent_pwmgr_form_autofill_result.list AS histogram_parent_pwmgr_form_autofill_result,
    histogram_parent_pwmgr_login_last_used_days.list AS histogram_parent_pwmgr_login_last_used_days,
    histogram_parent_pwmgr_login_page_safety.list AS histogram_parent_pwmgr_login_page_safety,
    histogram_parent_pwmgr_manage_opened.list AS histogram_parent_pwmgr_manage_opened,
    histogram_parent_pwmgr_manage_visibility_toggled.list AS histogram_parent_pwmgr_manage_visibility_toggled,
    histogram_parent_pwmgr_num_passwords_per_hostname.list AS histogram_parent_pwmgr_num_passwords_per_hostname,
    histogram_parent_pwmgr_num_saved_passwords.list AS histogram_parent_pwmgr_num_saved_passwords,
    histogram_parent_pwmgr_prompt_remember_action.list AS histogram_parent_pwmgr_prompt_remember_action,
    histogram_parent_pwmgr_prompt_update_action.list AS histogram_parent_pwmgr_prompt_update_action,
    histogram_parent_pwmgr_saving_enabled.list AS histogram_parent_pwmgr_saving_enabled,
    histogram_parent_sandbox_rejected_syscalls.list AS histogram_parent_sandbox_rejected_syscalls,
    histogram_parent_search_reset_result.list AS histogram_parent_search_reset_result,
    histogram_parent_search_service_init_ms.list AS histogram_parent_search_service_init_ms,
    histogram_parent_ssl_handshake_result.list AS histogram_parent_ssl_handshake_result,
    histogram_parent_ssl_handshake_version.list AS histogram_parent_ssl_handshake_version,
    histogram_parent_ssl_tls12_intolerance_reason_pre.list AS histogram_parent_ssl_tls12_intolerance_reason_pre,
    histogram_parent_ssl_tls13_intolerance_reason_pre.list AS histogram_parent_ssl_tls13_intolerance_reason_pre,
    histogram_parent_time_to_first_click_ms.list AS histogram_parent_time_to_first_click_ms,
    histogram_parent_time_to_first_interaction_ms.list AS histogram_parent_time_to_first_interaction_ms,
    histogram_parent_time_to_first_key_input_ms.list AS histogram_parent_time_to_first_key_input_ms,
    histogram_parent_time_to_first_mouse_move_ms.list AS histogram_parent_time_to_first_mouse_move_ms,
    histogram_parent_time_to_first_scroll_ms.list AS histogram_parent_time_to_first_scroll_ms,
    histogram_parent_time_to_non_blank_paint_ms.list AS histogram_parent_time_to_non_blank_paint_ms,
    histogram_parent_time_to_response_start_ms.list AS histogram_parent_time_to_response_start_ms,
    histogram_parent_touch_enabled_device.list AS histogram_parent_touch_enabled_device,
    histogram_parent_tracking_protection_enabled.list AS histogram_parent_tracking_protection_enabled,
    histogram_parent_update_bits_result_complete.list AS histogram_parent_update_bits_result_complete,
    histogram_parent_update_bits_result_partial.list AS histogram_parent_update_bits_result_partial,
    histogram_parent_update_can_use_bits_notify.list AS histogram_parent_update_can_use_bits_notify,
    histogram_parent_update_download_code_complete.list AS histogram_parent_update_download_code_complete,
    histogram_parent_update_download_code_partial.list AS histogram_parent_update_download_code_partial,
    histogram_parent_update_state_code_complete_stage.list AS histogram_parent_update_state_code_complete_stage,
    histogram_parent_update_state_code_complete_startup.list AS histogram_parent_update_state_code_complete_startup,
    histogram_parent_update_state_code_partial_stage.list AS histogram_parent_update_state_code_partial_stage,
    histogram_parent_update_state_code_partial_startup.list AS histogram_parent_update_state_code_partial_startup,
    histogram_parent_update_status_error_code_complete_stage.list AS histogram_parent_update_status_error_code_complete_stage,
    histogram_parent_update_status_error_code_complete_startup.list AS histogram_parent_update_status_error_code_complete_startup,
    histogram_parent_update_status_error_code_partial_stage.list AS histogram_parent_update_status_error_code_partial_stage,
    histogram_parent_update_status_error_code_partial_startup.list AS histogram_parent_update_status_error_code_partial_startup,
    histogram_parent_uptake_remote_content_result_1.list AS histogram_parent_uptake_remote_content_result_1,
    histogram_parent_webext_background_page_load_ms.list AS histogram_parent_webext_background_page_load_ms,
    histogram_parent_webext_browseraction_popup_open_ms.list AS histogram_parent_webext_browseraction_popup_open_ms,
    histogram_parent_webext_browseraction_popup_preload_result_count.list AS histogram_parent_webext_browseraction_popup_preload_result_count,
    histogram_parent_webext_content_script_injection_ms.list AS histogram_parent_webext_content_script_injection_ms,
    histogram_parent_webext_extension_startup_ms.list AS histogram_parent_webext_extension_startup_ms,
    histogram_parent_webext_pageaction_popup_open_ms.list AS histogram_parent_webext_pageaction_popup_open_ms,
    histogram_parent_webext_storage_local_get_ms.list AS histogram_parent_webext_storage_local_get_ms,
    histogram_parent_webext_storage_local_set_ms.list AS histogram_parent_webext_storage_local_set_ms,
    histogram_parent_webvr_time_spent_viewing_in_2d.list AS histogram_parent_webvr_time_spent_viewing_in_2d,
    histogram_parent_webvr_time_spent_viewing_in_oculus.list AS histogram_parent_webvr_time_spent_viewing_in_oculus,
    histogram_parent_webvr_time_spent_viewing_in_openvr.list AS histogram_parent_webvr_time_spent_viewing_in_openvr,
    histogram_parent_webvr_users_view_in.list AS histogram_parent_webvr_users_view_in
  )
FROM
  `moz-fx-data-shared-prod.telemetry_derived.main_summary_v4`
